"""Python decimal arithmetic module"""

# This is a slight lie, the implementations aren't exactly identical
# However, in all likelihood, the differences are inconsequential
import sys
from _decimal import *

__all__ = [
    "Decimal",
    "Context",
    "DecimalTuple",
    "DefaultContext",
    "BasicContext",
    "ExtendedContext",
    "DecimalException",
    "Clamped",
    "InvalidOperation",
    "DivisionByZero",
    "Inexact",
    "Rounded",
    "Subnormal",
    "Overflow",
    "Underflow",
    "FloatOperation",
    "DivisionImpossible",
    "InvalidContext",
    "ConversionSyntax",
    "DivisionUndefined",
    "ROUND_DOWN",
    "ROUND_HALF_UP",
    "ROUND_HALF_EVEN",
    "ROUND_CEILING",
    "ROUND_FLOOR",
    "ROUND_UP",
    "ROUND_HALF_DOWN",
    "ROUND_05UP",
    "setcontext",
    "getcontext",
    "localcontext",
    "MAX_PREC",
    "MAX_EMAX",
    "MIN_EMIN",
    "MIN_ETINY",
    "HAVE_THREADS",
    "HAVE_CONTEXTVAR",
]

if sys.version_info >= (3, 14):
    __all__ += ["IEEEContext", "IEEE_CONTEXT_MAX_BITS"]

if sys.version_info >= (3, 15):
    __all__ += ["SPEC_VERSION"]



import sys
from _typeshed import structseq
from collections.abc import Callable, Iterable
from enum import IntEnum
from types import FrameType
from typing import Any, Final, TypeAlias, final
from typing_extensions import Never

NSIG: int

class Signals(IntEnum):
    """An enumeration."""

    SIGFPE = 8
    SIGILL = 4
    SIGINT = 2
    SIGSEGV = 11
    SIGTERM = 15

    if sys.platform == "win32":
        SIGABRT = 22
        SIGBREAK = 21
        CTRL_C_EVENT = 0
        CTRL_BREAK_EVENT = 1
    else:
        SIGABRT = 6
        SIGALRM = 14
        SIGBUS = 7
        SIGCHLD = 17
        SIGCONT = 18
        SIGHUP = 1
        SIGIO = 29
        SIGIOT = 6
        SIGKILL = 9
        SIGPIPE = 13
        SIGPROF = 27
        SIGQUIT = 3
        SIGSTOP = 19
        SIGSYS = 31
        SIGTRAP = 5
        SIGTSTP = 20
        SIGTTIN = 21
        SIGTTOU = 22
        SIGURG = 23
        SIGUSR1 = 10
        SIGUSR2 = 12
        SIGVTALRM = 26
        SIGWINCH = 28
        SIGXCPU = 24
        SIGXFSZ = 25
        if sys.platform != "linux":
            SIGEMT = 7
            SIGINFO = 29
        if sys.platform != "darwin":
            SIGCLD = 17
            SIGPOLL = 29
            SIGPWR = 30
            SIGRTMAX = 64
            SIGRTMIN = 34
            if sys.version_info >= (3, 11):
                SIGSTKFLT = 16

class Handlers(IntEnum):
    """An enumeration."""

    SIG_DFL = 0
    SIG_IGN = 1

SIG_DFL: Final = Handlers.SIG_DFL
SIG_IGN: Final = Handlers.SIG_IGN

_SIGNUM: TypeAlias = int | Signals
_HANDLER: TypeAlias = Callable[[int, FrameType | None], Any] | int | Handlers | None

def default_int_handler(signalnum: int, frame: FrameType | None, /) -> Never:
    """The default handler for SIGINT installed by Python.

    It raises KeyboardInterrupt.
    """

def getsignal(signalnum: _SIGNUM) -> _HANDLER:
    """Return the current action for the given signal.

    The return value can be:
      SIG_IGN -- if the signal is being ignored
      SIG_DFL -- if the default action for the signal is in effect
      None    -- if an unknown handler is in effect
      anything else -- the callable Python object used as a handler
    """

def signal(signalnum: _SIGNUM, handler: _HANDLER) -> _HANDLER:
    """Set the action for the given signal.

    The action can be SIG_DFL, SIG_IGN, or a callable Python object.
    The previous action is returned.  See getsignal() for possible return
    values.

    *** IMPORTANT NOTICE ***
    A signal handler function is called with two arguments: the first is
    the signal number, the second is the interrupted stack frame.
    """

SIGABRT: Final = Signals.SIGABRT
SIGFPE: Final = Signals.SIGFPE
SIGILL: Final = Signals.SIGILL
SIGINT: Final = Signals.SIGINT
SIGSEGV: Final = Signals.SIGSEGV
SIGTERM: Final = Signals.SIGTERM

if sys.platform == "win32":
    SIGBREAK: Final = Signals.SIGBREAK
    CTRL_C_EVENT: Final = Signals.CTRL_C_EVENT
    CTRL_BREAK_EVENT: Final = Signals.CTRL_BREAK_EVENT
else:
    if sys.platform != "linux":
        SIGINFO: Final = Signals.SIGINFO
        SIGEMT: Final = Signals.SIGEMT
    SIGALRM: Final = Signals.SIGALRM
    SIGBUS: Final = Signals.SIGBUS
    SIGCHLD: Final = Signals.SIGCHLD
    SIGCONT: Final = Signals.SIGCONT
    SIGHUP: Final = Signals.SIGHUP
    SIGIO: Final = Signals.SIGIO
    SIGIOT: Final = Signals.SIGABRT  # alias
    SIGKILL: Final = Signals.SIGKILL
    SIGPIPE: Final = Signals.SIGPIPE
    SIGPROF: Final = Signals.SIGPROF
    SIGQUIT: Final = Signals.SIGQUIT
    SIGSTOP: Final = Signals.SIGSTOP
    SIGSYS: Final = Signals.SIGSYS
    SIGTRAP: Final = Signals.SIGTRAP
    SIGTSTP: Final = Signals.SIGTSTP
    SIGTTIN: Final = Signals.SIGTTIN
    SIGTTOU: Final = Signals.SIGTTOU
    SIGURG: Final = Signals.SIGURG
    SIGUSR1: Final = Signals.SIGUSR1
    SIGUSR2: Final = Signals.SIGUSR2
    SIGVTALRM: Final = Signals.SIGVTALRM
    SIGWINCH: Final = Signals.SIGWINCH
    SIGXCPU: Final = Signals.SIGXCPU
    SIGXFSZ: Final = Signals.SIGXFSZ

    class ItimerError(OSError): ...
    ITIMER_PROF: int
    ITIMER_REAL: int
    ITIMER_VIRTUAL: int

    class Sigmasks(IntEnum):
        """An enumeration."""

        SIG_BLOCK = 0
        SIG_UNBLOCK = 1
        SIG_SETMASK = 2

    SIG_BLOCK: Final = Sigmasks.SIG_BLOCK
    SIG_UNBLOCK: Final = Sigmasks.SIG_UNBLOCK
    SIG_SETMASK: Final = Sigmasks.SIG_SETMASK
    def alarm(seconds: int, /) -> int:
        """Arrange for SIGALRM to arrive after the given number of seconds."""

    def getitimer(which: int, /) -> tuple[float, float]:
        """Returns current value of given itimer."""

    def pause() -> None:
        """Wait until a signal arrives."""

    def pthread_kill(thread_id: int, signalnum: int, /) -> None:
        """Send a signal to a thread."""

    def pthread_sigmask(how: int, mask: Iterable[int]) -> set[_SIGNUM]:
        """Fetch and/or change the signal mask of the calling thread."""

    def setitimer(which: int, seconds: float, interval: float = 0.0, /) -> tuple[float, float]:
        """Sets given itimer (one of ITIMER_REAL, ITIMER_VIRTUAL or ITIMER_PROF).

        The timer will fire after value seconds and after that every interval
        seconds.  The itimer can be cleared by setting seconds to zero.

        Returns old values as a tuple: (delay, interval).
        """

    def siginterrupt(signalnum: int, flag: bool, /) -> None:
        """Change system call restart behaviour.

        If flag is False, system calls will be restarted when interrupted by
        signal sig, else system calls will be interrupted.
        """

    def sigpending() -> Any:
        """Examine pending signals.

        Returns a set of signal numbers that are pending for delivery to
        the calling thread.
        """

    def sigwait(sigset: Iterable[int]) -> _SIGNUM:
        """Wait for a signal.

        Suspend execution of the calling thread until the delivery of one of the
        signals specified in the signal set sigset.  The function accepts the
        signal and returns the signal number.
        """

    if sys.platform != "darwin":
        SIGCLD: Final = Signals.SIGCHLD  # alias
        SIGPOLL: Final = Signals.SIGIO  # alias
        SIGPWR: Final = Signals.SIGPWR
        SIGRTMAX: Final = Signals.SIGRTMAX
        SIGRTMIN: Final = Signals.SIGRTMIN
        if sys.version_info >= (3, 11):
            SIGSTKFLT: Final = Signals.SIGSTKFLT

        @final
        class struct_siginfo(structseq[int], tuple[int, int, int, int, int, int, int]):
            """struct_siginfo: Result from sigwaitinfo or sigtimedwait.

            This object may be accessed either as a tuple of
            (si_signo, si_code, si_errno, si_pid, si_uid, si_status, si_band),
            or via the attributes si_signo, si_code, and so on.
            """

            __match_args__: Final = ("si_signo", "si_code", "si_errno", "si_pid", "si_uid", "si_status", "si_band")

            @property
            def si_signo(self) -> int:
                """signal number"""

            @property
            def si_code(self) -> int:
                """signal code"""

            @property
            def si_errno(self) -> int:
                """errno associated with this signal"""

            @property
            def si_pid(self) -> int:
                """sending process ID"""

            @property
            def si_uid(self) -> int:
                """real user ID of sending process"""

            @property
            def si_status(self) -> int:
                """exit value or signal"""

            @property
            def si_band(self) -> int:
                """band event for SIGPOLL"""

        def sigtimedwait(sigset: Iterable[int], timeout: float, /) -> struct_siginfo | None:
            """Like sigwaitinfo(), but with a timeout.

            The timeout is specified in seconds, rounded up to nanoseconds.
            """

        def sigwaitinfo(sigset: Iterable[int], /) -> struct_siginfo:
            """Wait synchronously until one of the signals in *sigset* is delivered.

            Returns a struct_siginfo containing information about the signal.
            """

def strsignal(signalnum: _SIGNUM, /) -> str | None:
    """Return the system description of the given signal.

    Returns the description of signal *signalnum*, such as "Interrupt"
    for :const:`SIGINT`. Returns :const:`None` if *signalnum* has no
    description. Raises :exc:`ValueError` if *signalnum* is invalid.
    """

def valid_signals() -> set[Signals]:
    """Return a set of valid signal numbers on this platform.

    The signal numbers returned by this function can be safely passed to
    functions like `pthread_sigmask`.
    """

def raise_signal(signalnum: _SIGNUM, /) -> None:
    """Send a signal to the executing process."""

def set_wakeup_fd(fd: int, /, *, warn_on_full_buffer: bool = True) -> int:
    """Sets the fd to be written to (with the signal number) when a signal comes in.

    A library can use this to wakeup select or poll.
    The previous fd or -1 is returned.

    The fd must be non-blocking.
    """

if sys.platform == "linux":
    def pidfd_send_signal(pidfd: int, sig: int, siginfo: None = None, flags: int = 0, /) -> None:
        """Send a signal to a process referred to by a pid file descriptor."""



"""Generic interface to all dbm clones.

Use

        import dbm
        d = dbm.open(file, 'w', 0o666)

The returned object is a dbm.sqlite3, dbm.gnu, dbm.ndbm or dbm.dumb database object, dependent on the
type of database being opened (determined by the whichdb function) in the case
of an existing dbm. If the dbm does not exist and the create or new flag ('c'
or 'n') was specified, the dbm type will be determined by the availability of
the modules (tested in the above order).

It has the following interface (key and data are strings):

        d[key] = data   # store data at key (may override data at
                        # existing key)
        data = d[key]   # retrieve data at key (raise KeyError if no
                        # such key)
        del d[key]      # delete data stored at key (raises KeyError
                        # if no such key)
        flag = key in d # true if the key exists
        list = d.keys() # return a list of all existing keys (slow!)

Future versions may change the order in which implementations are
tested for existence, and add interfaces to other dbm-like
implementations.
"""

import sys
from _typeshed import StrOrBytesPath
from collections.abc import Iterator, MutableMapping
from types import TracebackType
from typing import Literal, TypeAlias, type_check_only
from typing_extensions import Self

__all__ = ["open", "whichdb", "error"]

_KeyType: TypeAlias = str | bytes
_ValueType: TypeAlias = str | bytes | bytearray
_TFlags: TypeAlias = Literal[
    "r",
    "w",
    "c",
    "n",
    "rf",
    "wf",
    "cf",
    "nf",
    "rs",
    "ws",
    "cs",
    "ns",
    "ru",
    "wu",
    "cu",
    "nu",
    "rfs",
    "wfs",
    "cfs",
    "nfs",
    "rfu",
    "wfu",
    "cfu",
    "nfu",
    "rsf",
    "wsf",
    "csf",
    "nsf",
    "rsu",
    "wsu",
    "csu",
    "nsu",
    "ruf",
    "wuf",
    "cuf",
    "nuf",
    "rus",
    "wus",
    "cus",
    "nus",
    "rfsu",
    "wfsu",
    "cfsu",
    "nfsu",
    "rfus",
    "wfus",
    "cfus",
    "nfus",
    "rsfu",
    "wsfu",
    "csfu",
    "nsfu",
    "rsuf",
    "wsuf",
    "csuf",
    "nsuf",
    "rufs",
    "wufs",
    "cufs",
    "nufs",
    "rusf",
    "wusf",
    "cusf",
    "nusf",
]

@type_check_only
class _Database(MutableMapping[_KeyType, bytes]):
    def close(self) -> None: ...
    def __getitem__(self, key: _KeyType) -> bytes: ...
    def __setitem__(self, key: _KeyType, value: _ValueType) -> None: ...
    def __delitem__(self, key: _KeyType) -> None: ...
    def __iter__(self) -> Iterator[bytes]: ...
    def __len__(self) -> int: ...
    def __del__(self) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...

# This class is not exposed. It calls itself dbm.error.
@type_check_only
class _error(Exception): ...

error: tuple[type[_error], type[OSError]]

if sys.version_info >= (3, 11):
    def whichdb(filename: StrOrBytesPath) -> str | None:
        """Guess which db package to use to open a db file.

        Return values:

        - None if the database file can't be read;
        - empty string if the file can be read but can't be recognized
        - the name of the dbm submodule (e.g. "ndbm" or "gnu") if recognized.

        Importing the given module may still fail, and opening the
        database using that module may still fail.
        """

    def open(file: StrOrBytesPath, flag: _TFlags = "r", mode: int = 0o666) -> _Database:
        """Open or create database at path given by *file*.

        Optional argument *flag* can be 'r' (default) for read-only access, 'w'
        for read-write access of an existing database, 'c' for read-write access
        to a new or existing database, and 'n' for read-write access to a new
        database.

        Note: 'r' and 'w' fail if the database doesn't exist; 'c' creates it
        only if it doesn't exist; and 'n' always creates a new database.
        """

else:
    def whichdb(filename: str) -> str | None:
        """Guess which db package to use to open a db file.

        Return values:

        - None if the database file can't be read;
        - empty string if the file can be read but can't be recognized
        - the name of the dbm submodule (e.g. "ndbm" or "gnu") if recognized.

        Importing the given module may still fail, and opening the
        database using that module may still fail.
        """

    def open(file: str, flag: _TFlags = "r", mode: int = 0o666) -> _Database:
        """Open or create database at path given by *file*.

        Optional argument *flag* can be 'r' (default) for read-only access, 'w'
        for read-write access of an existing database, 'c' for read-write access
        to a new or existing database, and 'n' for read-write access to a new
        database.

        Note: 'r' and 'w' fail if the database doesn't exist; 'c' creates it
        only if it doesn't exist; and 'n' always creates a new database.
        """



from _typeshed import Incomplete, StrOrBytesPath, StrPath
from collections.abc import Generator, Sequence
from tempfile import TemporaryDirectory
from typing import Any, ClassVar, Final, TypedDict, type_check_only

from _remote_debugging import AwaitedInfo, InterpreterInfo

from .collector import Collector, _Timestamps

@type_check_only
class _GeckoCategory(TypedDict):
    name: str
    color: str
    subcategories: list[str]

THREAD_STATUS_HAS_GIL: Final[int]
THREAD_STATUS_ON_CPU: Final[int]
THREAD_STATUS_UNKNOWN: Final[int]
THREAD_STATUS_GIL_REQUESTED: Final[int]
THREAD_STATUS_HAS_EXCEPTION: Final[int]
THREAD_STATUS_MAIN_THREAD: Final[int]

GECKO_CATEGORIES: Final[list[_GeckoCategory]]

CATEGORY_OTHER: Final = 0
CATEGORY_PYTHON: Final = 1
CATEGORY_NATIVE: Final = 2
CATEGORY_GC: Final = 3
CATEGORY_GIL: Final = 4
CATEGORY_CPU: Final = 5
CATEGORY_CODE_TYPE: Final = 6
CATEGORY_OPCODES: Final = 7
CATEGORY_EXCEPTION: Final = 8

DEFAULT_SUBCATEGORY: Final = 0

GECKO_FORMAT_VERSION: Final = 32
GECKO_PREPROCESSED_VERSION: Final = 57

RESOURCE_TYPE_LIBRARY: Final = 1

FRAME_ADDRESS_NONE: Final = -1
FRAME_INLINE_DEPTH_ROOT: Final = 0

PROCESS_TYPE_MAIN: Final = 0
STACKWALK_DISABLED: Final = 0

DEFAULT_SPILL_BUFFER_BYTES: Final[int]

class SpillColumn:
    path: str
    buffer: bytearray

    def __init__(self, directory: StrPath, basename: StrPath, *, buffer_bytes: int | None = None) -> None: ...
    # "value" accepts the same types as json.JSONEncoder.encode()
    def append(self, value: Any) -> None: ...
    def flush(self) -> None: ...
    def iter_tokens(self) -> Generator[str]: ...

class GeckoThreadSpill:
    sample_count: int
    marker_count: int
    def __init__(self, directory: StrPath, tid: int) -> None: ...
    def append_sample(self, stack_index: int, time_ms: float) -> None: ...
    def append_marker(
        self, name_idx: int, start_time: float, end_time: float, phase: int, category: int, data: dict[str, Any]
    ) -> None: ...
    def prepare_read(self) -> None: ...

class GeckoCollector(Collector):
    aggregating: ClassVar[bool]

    sample_interval_usec: int
    skip_idle: bool
    opcodes_enabled: bool
    start_time: float

    global_strings: list[str]
    global_string_map: dict[str, int]

    threads: dict[int, dict[str, Any]]
    spill_dir: TemporaryDirectory[str] | None
    exported: bool

    libs: list[Incomplete]

    sample_count: int
    last_sample_time: float
    interval: float

    has_gil_start: dict[Incomplete, Incomplete]
    no_gil_start: dict[Incomplete, Incomplete]
    on_cpu_start: dict[Incomplete, Incomplete]
    off_cpu_start: dict[Incomplete, Incomplete]
    python_code_start: dict[Incomplete, Incomplete]
    native_code_start: dict[Incomplete, Incomplete]
    gil_wait_start: dict[Incomplete, Incomplete]
    exception_start: dict[Incomplete, Incomplete]
    no_exception_start: dict[Incomplete, Incomplete]

    gc_start_per_thread: dict[int, float]

    initialized_threads: set[Incomplete]

    opcode_state: dict[int, tuple[Incomplete, int, int, str, str, float]]

    def __init__(self, sample_interval_usec: int, *, skip_idle: bool = False, opcodes: bool = False) -> None: ...
    def collect(self, stack_frames: Sequence[InterpreterInfo] | Sequence[AwaitedInfo], timestamps_us: _Timestamps = None) -> None:
        """Collect samples from stack frames.

        Args:
            stack_frames: List of interpreter/thread frame info
            timestamps_us: List of timestamps in microseconds (None for live sampling)
        """

    def export(self, filename: StrOrBytesPath) -> None:
        """Export the profile to a Gecko JSON file."""



def http_error(status):
    match status   : # fmt: skip
        case 400   : # fmt: skip
            return "Bad request"
        case 404:
            return "Not found"
        case 418:
            return "I'm a teapot"
        case _:
            return "Something's wrong with the internet"

# point is an (x, y) tuple
match point:
    case (0,   0): # fmt: skip
        print("Origin")
    case (0, y):
        print(f"Y={y}")
    case (x, 0):
        print(f"X={x}")
    case (x, y):
        print(f"X={x}, Y={y}")
    case _:
        raise ValueError("Not a point")

class Point:
    x: int
    y: int

def location(point):
    match point:
        case Point(x=0, y =0  ) : # fmt: skip
            print("Origin is the point's location.")
        case Point(x=0, y=y):
            print(f"Y={y} and the point is on the y-axis.")
        case Point(x=x, y=0):
            print(f"X={x} and the point is on the x-axis.")
        case Point():
            print("The point is located somewhere else on the plane.")
        case _:
            print("Not a point")


match points:
    case []:
        print("No points in the list.")
    case [
        Point(0, 0)
    ]: # fmt: skip
        print("The origin is the only point in the list.")
    case [Point(x, y)]:
        print(f"A single point {x}, {y} is in the list.")
    case [Point(0, y1), Point(0, y2)]:
        print(f"Two points on the Y axis at {y1}, {y2} are in the list.")
    case _:
        print("Something else is found in the list.")


match test_variable:
    case (
            'warning',
            code,
            40
    ): # fmt: skip
        print("A warning has been received.")
    case ('error', code, _):
        print(f"An error {code} occurred.")


match point:
    case Point(x, y) if x   == y:  # fmt: skip
        print(f"The point is located on the diagonal Y=X at {x}.")
    case Point(x, y):
        print(f"Point is not on the diagonal.")



"""
Compatibility shim for .resources.readers as found on Python 3.10.

Consumers that can rely on Python 3.11 should use the other
module directly.
"""

# On py311+, things are actually defined in importlib.resources.readers,
# and re-exported here,
# but doing it this way leads to less code duplication for us

import pathlib
import sys
import zipfile
from _typeshed import StrPath
from collections.abc import Iterable, Iterator
from importlib._bootstrap_external import FileLoader
from io import BufferedReader
from typing import Literal, TypeVar
from typing_extensions import Never
from zipimport import zipimporter

if sys.version_info >= (3, 11):
    from importlib.resources import abc
else:
    from importlib import abc

if sys.version_info >= (3, 11):
    __all__ = ["FileReader", "ZipReader", "MultiplexedPath", "NamespaceReader"]

if sys.version_info < (3, 11):
    _T = TypeVar("_T")

    def remove_duplicates(items: Iterable[_T]) -> Iterator[_T]: ...

class FileReader(abc.TraversableResources):
    path: pathlib.Path
    def __init__(self, loader: FileLoader) -> None: ...
    def resource_path(self, resource: StrPath) -> str:
        """
        Return the file system path to prevent
        `resources.path()` from creating a temporary
        copy.
        """

    def files(self) -> pathlib.Path: ...

class ZipReader(abc.TraversableResources):
    prefix: str
    archive: str
    def __init__(self, loader: zipimporter, module: str) -> None: ...
    def open_resource(self, resource: str) -> BufferedReader: ...
    def is_resource(self, path: StrPath) -> bool:
        """
        Workaround for `zipfile.Path.is_file` returning true
        for non-existent paths.
        """

    def files(self) -> zipfile.Path: ...

class MultiplexedPath(abc.Traversable):
    """
    Given a series of Traversable objects, implement a merged
    version of the interface across all objects. Useful for
    namespace packages which may be multihomed at a single
    name.
    """

    def __init__(self, *paths: abc.Traversable) -> None: ...
    def iterdir(self) -> Iterator[abc.Traversable]: ...
    def read_bytes(self) -> Never: ...
    def read_text(self, *args: Never, **kwargs: Never) -> Never: ...  # type: ignore[override]
    def is_dir(self) -> Literal[True]: ...
    def is_file(self) -> Literal[False]: ...

    if sys.version_info >= (3, 12):
        def joinpath(self, *descendants: StrPath) -> abc.Traversable: ...
    elif sys.version_info >= (3, 11):
        def joinpath(self, child: StrPath) -> abc.Traversable: ...  # type: ignore[override]
    else:
        def joinpath(self, child: str) -> abc.Traversable: ...

    if sys.version_info < (3, 12):
        __truediv__ = joinpath

    def open(self, *args: Never, **kwargs: Never) -> Never: ...  # type: ignore[override]
    @property
    def name(self) -> str: ...

class NamespaceReader(abc.TraversableResources):
    path: MultiplexedPath
    def __init__(self, namespace_path: Iterable[str]) -> None: ...
    def resource_path(self, resource: str) -> str:
        """
        Return the file system path to prevent
        `resources.path()` from creating a temporary
        copy.
        """

    def files(self) -> MultiplexedPath: ...



class Event:
    event_name: ClassVar[str]

    @staticmethod
    def cls_for(event_name: str) -> type[Event]:
        event_cls = _CONCRETE_EVENT_CLASSES.get(event_name)
        if event_cls is not <RANGE_START>None:
            return event_cls<RANGE_END>
        else:
            raise ValueError(f"unknown event name '{event_name}'")

class Event:
    event_name: ClassVar[str]

    @staticmethod
    def cls_for(event_name: str) -> type[Event]:
        event_cls = _CONCRETE_EVENT_CLASSES.get(event_name)
        if event_cls is not None:
            <RANGE_START>

            <RANGE_END>
            return event_cls
        else:
            raise ValueError(f"unknown event name '{event_name}'")


# The user starts adding items to a list and then hits save.
# Ruff should trim the empty lines
a = [
    1,
    2,
    3,<RANGE_START>


    <RANGE_END>
]

print("Don't format this"  )


# The user removed an argument from a call. Ruff should reformat the entire call
call(
    a,
    <RANGE_START>
    <RANGE_END>b,
    c,
    d
)

print("Don't format this"  )


#-----------------------------------------------------------------------------
# The user adds a new comment at the end:
<RANGE_START># <RANGE_END>
#-----------------------------------------------------------------------------

print("Don't format this"  )


def convert_str(value: str) -> str:  # Trailing comment
    """Return a string as-is."""

<RANGE_START>

    return value  # Trailing comment
<RANGE_END>
def test  ():
    pass


def test_comment_indent():
<RANGE_START># A misaligned comment<RANGE_END>
    print("test")


# This demonstrates the use case where a user inserts a new function or class after an existing function.
# In this case, we should avoid formatting the node that directly precedes the new function/class.
# However, the problem is that the preceding node **must** be formatted to determine the whitespace between the two statements.
def test_start ():
    print("Ideally this gets not reformatted" )

<RANGE_START>
def new_function_inserted_after_test_start ():
    print("This should get formatted" )<RANGE_END>



import sys
from _typeshed import StrPath
from abc import ABCMeta, abstractmethod
from collections.abc import Iterator
from io import BufferedReader
from typing import IO, Any, Literal, Protocol, overload, runtime_checkable
from typing_extensions import deprecated

if sys.version_info >= (3, 11):
    @deprecated("Deprecated. Use `importlib.resources.abc.TraversableResources` instead.")
    class ResourceReader(metaclass=ABCMeta):
        """Abstract base class for loaders to provide resource reading support."""

        @abstractmethod
        def open_resource(self, resource: str) -> IO[bytes]:
            """Return an opened, file-like object for binary reading.

            The 'resource' argument is expected to represent only a file name.
            If the resource cannot be found, FileNotFoundError is raised.
            """

        @abstractmethod
        def resource_path(self, resource: str) -> str:
            """Return the file system path to the specified resource.

            The 'resource' argument is expected to represent only a file name.
            If the resource does not exist on the file system, raise
            FileNotFoundError.
            """

        @abstractmethod
        def is_resource(self, path: str) -> bool:
            """Return True if the named 'path' is a resource.

            Files are resources, directories are not.
            """

        @abstractmethod
        def contents(self) -> Iterator[str]:
            """Return an iterable of entries in `package`."""

    @runtime_checkable
    class Traversable(Protocol):
        """
        An object with a subset of pathlib.Path methods suitable for
        traversing directories and opening files.

        Any exceptions that occur when accessing the backing resource
        may propagate unaltered.
        """

        @abstractmethod
        def is_dir(self) -> bool:
            """
            Return True if self is a directory
            """

        @abstractmethod
        def is_file(self) -> bool:
            """
            Return True if self is a file
            """

        @abstractmethod
        def iterdir(self) -> Iterator[Traversable]:
            """
            Yield Traversable objects in self
            """

        @abstractmethod
        def joinpath(self, *descendants: StrPath) -> Traversable:
            """
            Return Traversable resolved with any descendants applied.

            Each descendant should be a path segment relative to self
            and each may contain multiple levels separated by
            ``posixpath.sep`` (``/``).
            """

        # The documentation and runtime protocol allows *args, **kwargs arguments,
        # but this would mean that all implementers would have to support them,
        # which is not the case.
        @overload
        @abstractmethod
        def open(self, mode: Literal["r"] = "r", *, encoding: str | None = None, errors: str | None = None) -> IO[str]:
            """
            mode may be 'r' or 'rb' to open as text or binary. Return a handle
            suitable for reading (same as pathlib.Path.open).

            When opening as text, accepts encoding parameters such as those
            accepted by io.TextIOWrapper.
            """
        @overload
        @abstractmethod
        def open(self, mode: Literal["rb"]) -> IO[bytes]: ...

        @property
        @abstractmethod
        def name(self) -> str:
            """
            The base name of this object without any parent references.
            """

        def __truediv__(self, child: StrPath, /) -> Traversable:
            """
            Return Traversable child in self
            """

        @abstractmethod
        def read_bytes(self) -> bytes:
            """
            Read contents of self as bytes
            """

        if sys.version_info >= (3, 15):
            @abstractmethod
            def read_text(self, encoding: str | None = None, errors: str | None = None) -> str:
                """
                Read contents of self as text
                """
        else:
            @abstractmethod
            def read_text(self, encoding: str | None = None) -> str:
                """
                Read contents of self as text
                """

    class TraversableResources(ResourceReader):
        """
        The required interface for providing traversable
        resources.
        """

        @abstractmethod
        def files(self) -> Traversable:
            """Return a Traversable object for the loaded package."""

        def open_resource(self, resource: str) -> BufferedReader: ...
        def resource_path(self, resource: Any) -> str: ...
        def is_resource(self, path: str) -> bool: ...
        def contents(self) -> Iterator[str]: ...

    __all__ = ["ResourceReader", "Traversable", "TraversableResources"]



import sys
from _typeshed import StrPath
from collections.abc import Iterator
from os import PathLike
from typing import Any, Protocol, overload
from typing_extensions import TypeVar

_T = TypeVar("_T")
_T_co = TypeVar("_T_co", covariant=True, default=Any)

class PackageMetadata(Protocol):
    def __len__(self) -> int: ...
    def __contains__(self, item: str) -> bool: ...
    def __getitem__(self, key: str) -> str: ...
    def __iter__(self) -> Iterator[str]: ...
    @property
    def json(self) -> dict[str, str | list[str]]:
        """
        A JSON-compatible form of the metadata.
        """

    @overload
    def get_all(self, name: str, failobj: None = None) -> list[Any] | None:
        """Helper for @overload to raise when called."""
    @overload
    def get_all(self, name: str, failobj: _T) -> list[Any] | _T: ...

    if sys.version_info >= (3, 12):
        @overload
        def get(self, name: str, failobj: None = None) -> str | None:
            """Helper for @overload to raise when called."""
        @overload
        def get(self, name: str, failobj: _T) -> _T | str: ...

if sys.version_info >= (3, 13):
    class SimplePath(Protocol):
        """
        A minimal subset of pathlib.Path required by Distribution.
        """

        def joinpath(self, other: StrPath, /) -> SimplePath: ...
        def __truediv__(self, other: StrPath, /) -> SimplePath: ...
        # Incorrect at runtime
        @property
        def parent(self) -> PathLike[str]: ...
        def read_text(self, encoding: str | None = None) -> str: ...
        def read_bytes(self) -> bytes: ...
        def exists(self) -> bool: ...

elif sys.version_info >= (3, 12):
    class SimplePath(Protocol[_T_co]):
        """
        A minimal subset of pathlib.Path required by PathDistribution.
        """

        # At runtime this is defined as taking `str | _T`, but that causes trouble.
        # See #11436.
        def joinpath(self, other: str, /) -> _T_co: ...
        @property
        def parent(self) -> _T_co: ...
        def read_text(self) -> str: ...
        # As with joinpath(), this is annotated as taking `str | _T` at runtime.
        def __truediv__(self, other: str, /) -> _T_co: ...

else:
    class SimplePath(Protocol):
        """
        A minimal subset of pathlib.Path required by PathDistribution.
        """

        # Actually takes only self at runtime, but that's clearly wrong
        def joinpath(self, other: Any, /) -> SimplePath: ...
        # Not defined as a property at runtime, but it should be
        @property
        def parent(self) -> Any: ...
        def read_text(self) -> str: ...
        # There was a bug in `SimplePath` definition in cpython, see #8451
        #  Strictly speaking `__div__` was defined in 3.10, not __truediv__,
        # but it should have always been `__truediv__`.
        # Also, the runtime defines this method as taking no arguments,
        # which is obviously wrong.
        def __truediv__(self, other: Any, /) -> SimplePath: ...



# long variable name
this_is_a_ridiculously_long_name_and_nobody_in_their_right_mind_would_use_one_like_it = 0
this_is_a_ridiculously_long_name_and_nobody_in_their_right_mind_would_use_one_like_it = 1  # with a comment
this_is_a_ridiculously_long_name_and_nobody_in_their_right_mind_would_use_one_like_it = [
    1, 2, 3
]
this_is_a_ridiculously_long_name_and_nobody_in_their_right_mind_would_use_one_like_it = function()
this_is_a_ridiculously_long_name_and_nobody_in_their_right_mind_would_use_one_like_it = function(
    arg1, arg2, arg3
)
this_is_a_ridiculously_long_name_and_nobody_in_their_right_mind_would_use_one_like_it = function(
    [1, 2, 3], arg1, [1, 2, 3], arg2, [1, 2, 3], arg3
)
# long function name
normal_name = but_the_function_name_is_now_ridiculously_long_and_it_is_still_super_annoying()
normal_name = but_the_function_name_is_now_ridiculously_long_and_it_is_still_super_annoying(
    arg1, arg2, arg3
)
normal_name = but_the_function_name_is_now_ridiculously_long_and_it_is_still_super_annoying(
    [1, 2, 3], arg1, [1, 2, 3], arg2, [1, 2, 3], arg3
)
string_variable_name = (
    "a string that is waaaaaaaayyyyyyyy too long, even in parens, there's nothing you can do"  # noqa
)
for key in """
    hostname
    port
    username
""".split():
    if key in self.connect_kwargs:
        raise ValueError(err.format(key))
concatenated_strings = "some strings that are " "concatenated implicitly, so if you put them on separate " "lines it will fit"
del concatenated_strings, string_variable_name, normal_function_name, normal_name, need_more_to_make_the_line_long_enough
del ([], name_1, name_2), [(), [], name_4, name_3], name_1[[name_2 for name_1 in name_0]]
del (),



"""Introspection utils for tasks call graphs."""

import sys
from _typeshed import SupportsWrite
from asyncio import Future
from dataclasses import dataclass
from types import FrameType
from typing import Any, overload

if sys.version_info >= (3, 14):
    __all__ = ("capture_call_graph", "format_call_graph", "print_call_graph", "FrameCallGraphEntry", "FutureCallGraph")

    @dataclass(frozen=True, slots=True)
    class FrameCallGraphEntry:
        """FrameCallGraphEntry(frame: frame)"""

        frame: FrameType

    @dataclass(frozen=True, slots=True)
    class FutureCallGraph:
        """FutureCallGraph(future: _asyncio.Future, call_stack: tuple['FrameCallGraphEntry', ...], awaited_by: tuple['FutureCallGraph', ...])"""

        future: Future[Any]
        call_stack: tuple[FrameCallGraphEntry, ...]
        awaited_by: tuple[FutureCallGraph, ...]

    @overload
    def capture_call_graph(future: None = None, /, *, depth: int = 1, limit: int | None = None) -> FutureCallGraph | None:
        """Capture the async call graph for the current task or the provided Future.

        The graph is represented with three data structures:

        * FutureCallGraph(future, call_stack, awaited_by)

          Where 'future' is an instance of asyncio.Future or asyncio.Task.

          'call_stack' is a tuple of FrameGraphEntry objects.

          'awaited_by' is a tuple of FutureCallGraph objects.

        * FrameCallGraphEntry(frame)

          Where 'frame' is a frame object of a regular Python function
          in the call stack.

        Receives an optional 'future' argument. If not passed,
        the current task will be used. If there's no current task, the function
        returns None.

        If "capture_call_graph()" is introspecting *the current task*, the
        optional keyword-only 'depth' argument can be used to skip the specified
        number of frames from top of the stack.

        If the optional keyword-only 'limit' argument is provided, each call
        stack in the resulting graph is truncated to include at most
        ``abs(limit)`` entries.  If 'limit' is positive, the entries left are
        the closest to the invocation point.  If 'limit' is negative, the
        topmost entries are left.  If 'limit' is omitted or None, all entries
        are present.  If 'limit' is 0, the call stack is not captured at all,
        only "awaited by" information is present.
        """
    @overload
    def capture_call_graph(future: Future[Any], /, *, depth: int = 1, limit: int | None = None) -> FutureCallGraph | None: ...

    def format_call_graph(future: Future[Any] | None = None, /, *, depth: int = 1, limit: int | None = None) -> str:
        """Return the async call graph as a string for `future`.

        If `future` is not provided, format the call graph for the current task.
        """

    def print_call_graph(
        future: Future[Any] | None = None, /, *, file: SupportsWrite[str] | None = None, depth: int = 1, limit: int | None = None
    ) -> None:
        """Print the async call graph for the current task or the provided Future."""



"""Parser engine for the grammar tables generated by pgen.

The grammar table must be loaded first.

See Parser/parser.c in the Python distribution for additional info on
how this parsing engine works.

"""

from _typeshed import Incomplete
from collections.abc import Sequence
from typing import TypeAlias

from ..pytree import _NL, _RawNode
from . import _Convert
from .grammar import _DFAS, Grammar

_Context: TypeAlias = Sequence[Incomplete]

class ParseError(Exception):
    """Exception to signal the parser is stuck."""

    msg: str
    type: int
    value: str | None
    context: _Context
    def __init__(self, msg: str, type: int, value: str | None, context: _Context) -> None: ...

class Parser:
    """Parser engine.

    The proper usage sequence is:

    p = Parser(grammar, [converter])  # create instance
    p.setup([start])                  # prepare for parsing
    <for each input token>:
        if p.addtoken(...):           # parse a token; may raise ParseError
            break
    root = p.rootnode                 # root of abstract syntax tree

    A Parser instance may be reused by calling setup() repeatedly.

    A Parser instance contains state pertaining to the current token
    sequence, and should not be used concurrently by different threads
    to parse separate token sequences.

    See driver.py for how to get input tokens by tokenizing a file or
    string.

    Parsing is complete when addtoken() returns True; the root of the
    abstract syntax tree can then be retrieved from the rootnode
    instance variable.  When a syntax error occurs, addtoken() raises
    the ParseError exception.  There is no error recovery; the parser
    cannot be used after a syntax error was reported (but it can be
    reinitialized by calling setup()).

    """

    grammar: Grammar
    convert: _Convert
    stack: list[tuple[_DFAS, int, _RawNode]]
    rootnode: _NL | None
    used_names: set[str]
    def __init__(self, grammar: Grammar, convert: _Convert | None = None) -> None:
        """Constructor.

        The grammar argument is a grammar.Grammar instance; see the
        grammar module for more information.

        The parser is not ready yet for parsing; you must call the
        setup() method to get it started.

        The optional convert argument is a function mapping concrete
        syntax tree nodes to abstract syntax tree nodes.  If not
        given, no conversion is done and the syntax tree produced is
        the concrete syntax tree.  If given, it must be a function of
        two arguments, the first being the grammar (a grammar.Grammar
        instance), and the second being the concrete syntax tree node
        to be converted.  The syntax tree is converted from the bottom
        up.

        A concrete syntax tree node is a (type, value, context, nodes)
        tuple, where type is the node type (a token or symbol number),
        value is None for symbols and a string for tokens, context is
        None or an opaque value used for error reporting (typically a
        (lineno, offset) pair), and nodes is a list of children for
        symbols, and None for tokens.

        An abstract syntax tree node may be anything; this is entirely
        up to the converter function.

        """

    def setup(self, start: int | None = None) -> None:
        """Prepare for parsing.

        This *must* be called before starting to parse.

        The optional argument is an alternative start symbol; it
        defaults to the grammar's start symbol.

        You can use a Parser instance to parse any number of programs;
        each time you call setup() the parser is reset to an initial
        state determined by the (implicit or explicit) start symbol.

        """

    def addtoken(self, type: int, value: str | None, context: _Context) -> bool:
        """Add a token; return True iff this is the end of the program."""

    def classify(self, type: int, value: str | None, context: _Context) -> int:
        """Turn a token into a label.  (Internal)"""

    def shift(self, type: int, value: str | None, newstate: int, context: _Context) -> None:
        """Shift a token.  (Internal)"""

    def push(self, type: int, newdfa: _DFAS, newstate: int, context: _Context) -> None:
        """Push a nonterminal.  (Internal)"""

    def pop(self) -> None:
        """Pop a nonterminal.  (Internal)"""



"""A bottom-up tree matching algorithm implementation meant to speed
up 2to3's matching process. After the tree patterns are reduced to
their rarest linear path, a linear Aho-Corasick automaton is
created. The linear automaton traverses the linear paths from the
leaves to the root of the AST and returns a set of nodes for further
matching. This reduces significantly the number of candidate nodes.
"""

from _typeshed import Incomplete, SupportsGetItem
from collections import defaultdict
from collections.abc import Iterable

from .fixer_base import BaseFix
from .pytree import Leaf, Node

class BMNode:
    """Class for a node of the Aho-Corasick automaton used in matching"""

    count: Incomplete
    transition_table: Incomplete
    fixers: Incomplete
    id: Incomplete
    content: str
    def __init__(self) -> None: ...

class BottomMatcher:
    """The main matcher class. After instantiating the patterns should
    be added using the add_fixer method
    """

    match: Incomplete
    root: Incomplete
    nodes: Incomplete
    fixers: Incomplete
    logger: Incomplete
    def __init__(self) -> None: ...
    def add_fixer(self, fixer: BaseFix) -> None:
        """Reduces a fixer's pattern tree to a linear path and adds it
        to the matcher(a common Aho-Corasick automaton). The fixer is
        appended on the matching states and called when they are
        reached
        """

    def add(self, pattern: SupportsGetItem[int | slice, Incomplete] | None, start: BMNode) -> list[BMNode]:
        """Recursively adds a linear pattern to the AC automaton"""

    def run(self, leaves: Iterable[Leaf]) -> defaultdict[BaseFix, list[Node | Leaf]]:
        """The main interface with the bottom matcher. The tree is
        traversed from the bottom using the constructed
        automaton. Nodes are only checked once as the tree is
        retraversed. When the automaton fails, we give it one more
        shot(in case the above tree matches as a whole with the
        rejected leaf), then we break for the next leaf. There is the
        special case of multiple arguments(see code comments) where we
        recheck the nodes

        Args:
           The leaves of the AST tree to be matched

        Returns:
           A dictionary of node matches with fixers as the keys
        """

    def print_ac(self) -> None:
        """Prints a graphviz diagram of the BM automaton(for debugging)"""

def type_repr(type_num: int) -> str | int: ...



# Unparenthesized walruses are now allowed in set literals & set comprehensions
# since Python 3.9
{x := 1, 2, 3}
{x4 := x**5 for x in range(7)}
# We better not remove the parentheses here (since it's a 3.10 feature)
x[(a := 1)]
x[(a := 1), (b := 3)]



"""Selectors module.

This module allows high-level and efficient I/O multiplexing, built upon the
`select` module primitives.
"""

import sys
from _typeshed import FileDescriptor, FileDescriptorLike, Unused
from abc import ABCMeta, abstractmethod
from collections.abc import Mapping
from typing import Any, Final, NamedTuple
from typing_extensions import Self

EVENT_READ: Final = 1
EVENT_WRITE: Final = 2

class SelectorKey(NamedTuple):
    """SelectorKey(fileobj, fd, events, data)

    Object used to associate a file object to its backing
    file descriptor, selected event mask, and attached data.
    """

    fileobj: FileDescriptorLike
    fd: FileDescriptor
    events: int
    data: Any

class BaseSelector(metaclass=ABCMeta):
    """Selector abstract base class.

    A selector supports registering file objects to be monitored for specific
    I/O events.

    A file object is a file descriptor or any object with a `fileno()` method.
    An arbitrary object can be attached to the file object, which can be used
    for example to store context information, a callback, etc.

    A selector can use various implementations (select(), poll(), epoll()...)
    depending on the platform. The default `Selector` class uses the most
    efficient implementation on the current platform.
    """

    @abstractmethod
    def register(self, fileobj: FileDescriptorLike, events: int, data: Any = None) -> SelectorKey:
        """Register a file object.

        Parameters:
        fileobj -- file object or file descriptor
        events  -- events to monitor (bitwise mask of EVENT_READ|EVENT_WRITE)
        data    -- attached data

        Returns:
        SelectorKey instance

        Raises:
        ValueError if events is invalid
        KeyError if fileobj is already registered
        OSError if fileobj is closed or otherwise is unacceptable to
                the underlying system call (if a system call is made)

        Note:
        OSError may or may not be raised
        """

    @abstractmethod
    def unregister(self, fileobj: FileDescriptorLike) -> SelectorKey:
        """Unregister a file object.

        Parameters:
        fileobj -- file object or file descriptor

        Returns:
        SelectorKey instance

        Raises:
        KeyError if fileobj is not registered

        Note:
        If fileobj is registered but has since been closed this does
        *not* raise OSError (even if the wrapped syscall does)
        """

    def modify(self, fileobj: FileDescriptorLike, events: int, data: Any = None) -> SelectorKey:
        """Change a registered file object monitored events or attached data.

        Parameters:
        fileobj -- file object or file descriptor
        events  -- events to monitor (bitwise mask of EVENT_READ|EVENT_WRITE)
        data    -- attached data

        Returns:
        SelectorKey instance

        Raises:
        Anything that unregister() or register() raises
        """

    @abstractmethod
    def select(self, timeout: float | None = None) -> list[tuple[SelectorKey, int]]:
        """Perform the actual selection, until some monitored file objects are
        ready or a timeout expires.

        Parameters:
        timeout -- if timeout > 0, this specifies the maximum wait time, in
                   seconds
                   if timeout <= 0, the select() call won't block, and will
                   report the currently ready file objects
                   if timeout is None, select() will block until a monitored
                   file object becomes ready

        Returns:
        list of (key, events) for ready file objects
        `events` is a bitwise mask of EVENT_READ|EVENT_WRITE
        """

    def close(self) -> None:
        """Close the selector.

        This must be called to make sure that any underlying resource is freed.
        """

    def get_key(self, fileobj: FileDescriptorLike) -> SelectorKey:
        """Return the key associated to a registered file object.

        Returns:
        SelectorKey for this file object
        """

    @abstractmethod
    def get_map(self) -> Mapping[FileDescriptorLike, SelectorKey]:
        """Return a mapping of file objects to selector keys."""

    def __enter__(self) -> Self: ...
    def __exit__(self, *args: Unused) -> None: ...

class _BaseSelectorImpl(BaseSelector, metaclass=ABCMeta):
    """Base selector implementation."""

    def register(self, fileobj: FileDescriptorLike, events: int, data: Any = None) -> SelectorKey: ...
    def unregister(self, fileobj: FileDescriptorLike) -> SelectorKey: ...
    def modify(self, fileobj: FileDescriptorLike, events: int, data: Any = None) -> SelectorKey: ...
    def get_map(self) -> Mapping[FileDescriptorLike, SelectorKey]: ...

class SelectSelector(_BaseSelectorImpl):
    """Select-based selector."""

    def select(self, timeout: float | None = None) -> list[tuple[SelectorKey, int]]: ...

class _PollLikeSelector(_BaseSelectorImpl):
    """Base class shared between poll, epoll and devpoll selectors."""

    def select(self, timeout: float | None = None) -> list[tuple[SelectorKey, int]]: ...

if sys.platform != "win32":
    class PollSelector(_PollLikeSelector):
        """Poll-based selector."""

if sys.platform == "linux":
    class EpollSelector(_PollLikeSelector):
        """Epoll-based selector."""

        def fileno(self) -> int: ...

if sys.platform != "linux" and sys.platform != "darwin" and sys.platform != "win32":
    # Solaris only
    class DevpollSelector(_PollLikeSelector):
        def fileno(self) -> int: ...

if sys.platform != "win32" and sys.platform != "linux":
    class KqueueSelector(_BaseSelectorImpl):
        """Kqueue-based selector."""

        def fileno(self) -> int: ...
        def select(self, timeout: float | None = None) -> list[tuple[SelectorKey, int]]: ...

# Not a real class at runtime, it is just a conditional alias to other real selectors.
# The runtime logic is more fine-grained than a `sys.platform` check;
# not really expressible in the stubs
class DefaultSelector(_BaseSelectorImpl):
    """Epoll-based selector."""

    def select(self, timeout: float | None = None) -> list[tuple[SelectorKey, int]]: ...
    if sys.platform != "win32":
        def fileno(self) -> int: ...



"""Create portable serialized representations of Python objects.

See module copyreg for a mechanism for registering custom picklers.
See module pickletools source for extensive comments.

Classes:

    Pickler
    Unpickler

Functions:

    dump(object, file)
    dumps(object) -> string
    load(file) -> object
    loads(bytes) -> object

Misc variables:

    format_version
    compatible_formats

"""

import sys
from _pickle import (
    PickleError as PickleError,
    Pickler as Pickler,
    PicklingError as PicklingError,
    Unpickler as Unpickler,
    UnpicklingError as UnpicklingError,
    _BufferCallback,
    _ReadableFileobj,
    _ReducedType,
    dump as dump,
    dumps as dumps,
    load as load,
    loads as loads,
)
from _typeshed import ReadableBuffer, SupportsWrite
from collections.abc import Callable, Iterable, Mapping
from typing import Any, ClassVar, Final, SupportsBytes, SupportsIndex, final
from typing_extensions import Self

__all__ = [
    "PickleBuffer",
    "PickleError",
    "PicklingError",
    "UnpicklingError",
    "Pickler",
    "Unpickler",
    "dump",
    "dumps",
    "load",
    "loads",
    "ADDITEMS",
    "APPEND",
    "APPENDS",
    "BINBYTES",
    "BINBYTES8",
    "BINFLOAT",
    "BINGET",
    "BININT",
    "BININT1",
    "BININT2",
    "BINPERSID",
    "BINPUT",
    "BINSTRING",
    "BINUNICODE",
    "BINUNICODE8",
    "BUILD",
    "BYTEARRAY8",
    "DEFAULT_PROTOCOL",
    "DICT",
    "DUP",
    "EMPTY_DICT",
    "EMPTY_LIST",
    "EMPTY_SET",
    "EMPTY_TUPLE",
    "EXT1",
    "EXT2",
    "EXT4",
    "FALSE",
    "FLOAT",
    "FRAME",
    "FROZENSET",
    "GET",
    "GLOBAL",
    "HIGHEST_PROTOCOL",
    "INST",
    "INT",
    "LIST",
    "LONG",
    "LONG1",
    "LONG4",
    "LONG_BINGET",
    "LONG_BINPUT",
    "MARK",
    "MEMOIZE",
    "NEWFALSE",
    "NEWOBJ",
    "NEWOBJ_EX",
    "NEWTRUE",
    "NEXT_BUFFER",
    "NONE",
    "OBJ",
    "PERSID",
    "POP",
    "POP_MARK",
    "PROTO",
    "PUT",
    "READONLY_BUFFER",
    "REDUCE",
    "SETITEM",
    "SETITEMS",
    "SHORT_BINBYTES",
    "SHORT_BINSTRING",
    "SHORT_BINUNICODE",
    "STACK_GLOBAL",
    "STOP",
    "STRING",
    "TRUE",
    "TUPLE",
    "TUPLE1",
    "TUPLE2",
    "TUPLE3",
    "UNICODE",
]

HIGHEST_PROTOCOL: Final = 5
if sys.version_info >= (3, 14):
    DEFAULT_PROTOCOL: Final = 5
else:
    DEFAULT_PROTOCOL: Final = 4

bytes_types: tuple[type[Any], ...]  # undocumented

@final
class PickleBuffer:
    """Wrapper for potentially out-of-band buffers"""

    def __new__(cls, buffer: ReadableBuffer) -> Self: ...
    def raw(self) -> memoryview:
        """Return a memoryview of the raw memory underlying this buffer.
        Will raise BufferError is the buffer isn't contiguous.
        """

    def release(self) -> None:
        """Release the underlying buffer exposed by the PickleBuffer object."""

    def __buffer__(self, flags: int, /) -> memoryview:
        """Return a buffer object that exposes the underlying memory of the object."""

    def __release_buffer__(self, buffer: memoryview, /) -> None:
        """Release the buffer object that exposes the underlying memory of the object."""

MARK: Final = b"("
STOP: Final = b"."
POP: Final = b"0"
POP_MARK: Final = b"1"
DUP: Final = b"2"
FLOAT: Final = b"F"
INT: Final = b"I"
BININT: Final = b"J"
BININT1: Final = b"K"
LONG: Final = b"L"
BININT2: Final = b"M"
NONE: Final = b"N"
PERSID: Final = b"P"
BINPERSID: Final = b"Q"
REDUCE: Final = b"R"
STRING: Final = b"S"
BINSTRING: Final = b"T"
SHORT_BINSTRING: Final = b"U"
UNICODE: Final = b"V"
BINUNICODE: Final = b"X"
APPEND: Final = b"a"
BUILD: Final = b"b"
GLOBAL: Final = b"c"
DICT: Final = b"d"
EMPTY_DICT: Final = b"}"
APPENDS: Final = b"e"
GET: Final = b"g"
BINGET: Final = b"h"
INST: Final = b"i"
LONG_BINGET: Final = b"j"
LIST: Final = b"l"
EMPTY_LIST: Final = b"]"
OBJ: Final = b"o"
PUT: Final = b"p"
BINPUT: Final = b"q"
LONG_BINPUT: Final = b"r"
SETITEM: Final = b"s"
TUPLE: Final = b"t"
EMPTY_TUPLE: Final = b")"
SETITEMS: Final = b"u"
BINFLOAT: Final = b"G"

TRUE: Final = b"I01\n"
FALSE: Final = b"I00\n"

# protocol 2
PROTO: Final = b"\x80"
NEWOBJ: Final = b"\x81"
EXT1: Final = b"\x82"
EXT2: Final = b"\x83"
EXT4: Final = b"\x84"
TUPLE1: Final = b"\x85"
TUPLE2: Final = b"\x86"
TUPLE3: Final = b"\x87"
NEWTRUE: Final = b"\x88"
NEWFALSE: Final = b"\x89"
LONG1: Final = b"\x8a"
LONG4: Final = b"\x8b"

# protocol 3
BINBYTES: Final = b"B"
SHORT_BINBYTES: Final = b"C"

# protocol 4
SHORT_BINUNICODE: Final = b"\x8c"
BINUNICODE8: Final = b"\x8d"
BINBYTES8: Final = b"\x8e"
EMPTY_SET: Final = b"\x8f"
ADDITEMS: Final = b"\x90"
FROZENSET: Final = b"\x91"
NEWOBJ_EX: Final = b"\x92"
STACK_GLOBAL: Final = b"\x93"
MEMOIZE: Final = b"\x94"
FRAME: Final = b"\x95"

# protocol 5
BYTEARRAY8: Final = b"\x96"
NEXT_BUFFER: Final = b"\x97"
READONLY_BUFFER: Final = b"\x98"

def encode_long(x: int) -> bytes:  # undocumented
    """Encode a long to a two's complement little-endian binary string.
    Note that 0 is a special case, returning an empty string, to save a
    byte in the LONG1 pickling context.

    >>> encode_long(0)
    b''
    >>> encode_long(255)
    b'\\xff\\x00'
    >>> encode_long(32767)
    b'\\xff\\x7f'
    >>> encode_long(-256)
    b'\\x00\\xff'
    >>> encode_long(-32768)
    b'\\x00\\x80'
    >>> encode_long(-128)
    b'\\x80'
    >>> encode_long(127)
    b'\\x7f'
    >>>
    """

def decode_long(data: Iterable[SupportsIndex] | SupportsBytes | ReadableBuffer) -> int:  # undocumented
    """Decode a long from a two's complement little-endian binary string.

    >>> decode_long(b'')
    0
    >>> decode_long(b"\\xff\\x00")
    255
    >>> decode_long(b"\\xff\\x7f")
    32767
    >>> decode_long(b"\\x00\\xff")
    -256
    >>> decode_long(b"\\x00\\x80")
    -32768
    >>> decode_long(b"\\x80")
    -128
    >>> decode_long(b"\\x7f")
    127
    """

# undocumented pure-Python implementations
class _Pickler:
    fast: bool
    dispatch_table: Mapping[type, Callable[[Any], _ReducedType]]
    bin: bool  # undocumented
    dispatch: ClassVar[dict[type, Callable[[Unpickler, Any], None]]]  # undocumented, _Pickler only
    def __init__(
        self,
        file: SupportsWrite[bytes],
        protocol: int | None = None,
        *,
        fix_imports: bool = True,
        buffer_callback: _BufferCallback = None,
    ) -> None:
        """This takes a binary file for writing a pickle data stream.

        The optional *protocol* argument tells the pickler to use the
        given protocol; supported protocols are 0, 1, 2, 3, 4 and 5.
        The default protocol is 5. It was introduced in Python 3.8, and
        is incompatible with previous versions.

        Specifying a negative protocol version selects the highest
        protocol version supported.  The higher the protocol used, the
        more recent the version of Python needed to read the pickle
        produced.

        The *file* argument must have a write() method that accepts a
        single bytes argument. It can thus be a file object opened for
        binary writing, an io.BytesIO instance, or any other custom
        object that meets this interface.

        If *fix_imports* is True and *protocol* is less than 3, pickle
        will try to map the new Python 3 names to the old module names
        used in Python 2, so that the pickle data stream is readable
        with Python 2.

        If *buffer_callback* is None (the default), buffer views are
        serialized into *file* as part of the pickle stream.

        If *buffer_callback* is not None, then it can be called any number
        of times with a buffer view.  If the callback returns a false value
        (such as None), the given buffer is out-of-band; otherwise the
        buffer is serialized in-band, i.e. inside the pickle stream.

        It is an error if *buffer_callback* is not None and *protocol*
        is None or smaller than 5.
        """

    def dump(self, obj: Any) -> None:
        """Write a pickled representation of obj to the open file."""

    def clear_memo(self) -> None:
        """Clears the pickler's "memo".

        The memo is the data structure that remembers which objects the
        pickler has already seen, so that shared or recursive objects
        are pickled by reference and not by value.  This method is
        useful when re-using picklers.
        """

    def persistent_id(self, obj: Any) -> Any: ...
    # The following method is not defined on _Pickler, but can be defined on
    # sub-classes. Should return `NotImplemented` if pickling the supplied
    # object is not supported and returns the same types as `__reduce__()`.
    def reducer_override(self, obj: object, /) -> _ReducedType: ...

class _Unpickler:
    dispatch: ClassVar[dict[int, Callable[[Unpickler], None]]]  # undocumented, _Unpickler only
    def __init__(
        self,
        file: _ReadableFileobj,
        *,
        fix_imports: bool = True,
        encoding: str = "ASCII",
        errors: str = "strict",
        buffers: Iterable[Any] | None = None,
    ) -> None:
        """This takes a binary file for reading a pickle data stream.

        The protocol version of the pickle is detected automatically, so
        no proto argument is needed.

        The argument *file* must have two methods, a read() method that
        takes an integer argument, and a readline() method that requires
        no arguments.  Both methods should return bytes.  Thus *file*
        can be a binary file object opened for reading, an io.BytesIO
        object, or any other custom object that meets this interface.

        The file-like object must have two methods, a read() method
        that takes an integer argument, and a readline() method that
        requires no arguments.  Both methods should return bytes.
        Thus file-like object can be a binary file object opened for
        reading, a BytesIO object, or any other custom object that
        meets this interface.

        If *buffers* is not None, it should be an iterable of buffer-enabled
        objects that is consumed each time the pickle stream references
        an out-of-band buffer view.  Such buffers have been given in order
        to the *buffer_callback* of a Pickler object.

        If *buffers* is None (the default), then the buffers are taken
        from the pickle stream, assuming they are serialized there.
        It is an error for *buffers* to be None if the pickle stream
        was produced with a non-None *buffer_callback*.

        Other optional arguments are *fix_imports*, *encoding* and
        *errors*, which are used to control compatibility support for
        pickle stream generated by Python 2.  If *fix_imports* is True,
        pickle will try to map the old Python 2 names to the new names
        used in Python 3.  The *encoding* and *errors* tell pickle how
        to decode 8-bit string instances pickled by Python 2; these
        default to 'ASCII' and 'strict', respectively. *encoding* can be
        'bytes' to read these 8-bit string instances as bytes objects.
        """

    def load(self) -> Any:
        """Read a pickled object representation from the open file.

        Return the reconstituted object hierarchy specified in the file.
        """

    def find_class(self, module: str, name: str) -> Any: ...
    def persistent_load(self, pid: Any) -> Any: ...



# ruff: noqa: PYI021
"""
Internal-only symbols for special forms and type-system tests.

Some symbols provide definitions and on-hover documentation for special forms. Others exist only as
helpers for ty's tests. None of these symbols are intended to be directly imported by end users.
"""

import types
from enum import Enum
from typing import Any, Protocol, _SpecialForm

from typing_extensions import LiteralString, Self, TypeForm  # noqa: UP035

# -------------
# Special forms
# -------------

TypeOf: _SpecialForm
"""
`TypeOf[expression]` is the inferred type of `expression`.

Unlike a regular [type expression], the argument to `TypeOf` is interpreted as a
["value expression"][value expression]: an ordinary Python expression whose type ty infers. Whereas
`str` in a type annotation means "any instance of the class `str`", `TypeOf[str]` in a type annotation
signifies "the type inhabited by the `str` class object itself at runtime". `Literal[3]` is therefore
the same type as `TypeOf[3]`, since ty infers the object `3` as having type `Literal[3]`.

[type expression]: https://typing.python.org/en/latest/spec/annotations.html#type-and-annotation-expressions
[value expression]: https://docs.python.org/3/reference/expressions.html
"""

CallableTypeOf: _SpecialForm
"""
`CallableTypeOf[T]` extracts the callable type of `T` while preserving any function-like
behavior.

This means the result may behave differently from a plain `typing.Callable` in
type-theoretic checks. In particular, method-like and descriptor-like callables remain
distinct from regular callables.

Use this when you want to preserve the full callable flavor of a function, method, or
synthesized callable.
"""

RegularCallableTypeOf: _SpecialForm
"""
`RegularCallableTypeOf[T]` extracts a regular `typing.Callable`-style type from `T`.

This keeps the callable signatures of `T` but discards function-like behavior such as
descriptor-style method binding. Use this when you want to compare a callable against
ordinary `Callable[...]` types in type-theoretic tests.
"""

# -----
# Types
# -----

Unknown: _SpecialForm
"""
`Unknown` is a dynamic type inferred due to missing type information or an inference error.

ty infers `Unknown` for unannotated values with insufficient type information. It also uses it as a
fallback after certain type errors. This contrasts with `Any`, which represents an *explicitly*
annotated dynamic type. Like `Any`, however, it is a dynamic type, so ty allows any operation on it.
"""

Todo: _SpecialForm
"""
`@Todo` is a dynamic type inferred due to a known missing feature or incomplete implementation in
ty.

Like `Any` and `Unknown`, `@Todo` is a dynamic type, so ty allows any operation on it. Unlike `Any`,
it is not explicitly provided in an annotation; unlike `Unknown`, it specifically indicates a
limitation in ty.

It is an internal type used by ty and cannot be used in annotations. These types should disappear
as ty implements the missing functionality.
"""

Divergent: _SpecialForm
"""
`Divergent` is a dynamic type inferred due to type-level recursion that does not converge.

Type inference can be recursive. ty analyzes inference cycles repeatedly, looking for a
stable result. If each iteration produces a new type, ty replaces the non-convergent part
with `Divergent`.

Like `Any` and `Unknown`, `Divergent` is a dynamic type, so ty allows any operation on it.
Unlike `Unknown`, it does not represent missing type information. It is an internal type
used by ty and cannot be used in annotations.
"""

# -----------
# Constraints
# -----------

class ConstraintSetSolution:
    """One solution path for a constraint set."""

class ConstraintSet:
    @staticmethod
    def lower_bound(
        lower_bound: TypeForm[object],
        typevar: TypeForm[object],
    ) -> ConstraintSet:
        """Returns a constraint set requiring `typevar` to be a supertype of `lower_bound`."""

    @staticmethod
    def upper_bound(
        typevar: TypeForm[object],
        upper_bound: TypeForm[object],
    ) -> ConstraintSet:
        """Returns a constraint set requiring `typevar` to be a subtype of `upper_bound`."""

    @staticmethod
    def equality(
        typevar: TypeForm[object],
        value: TypeForm[object],
    ) -> ConstraintSet:
        """Returns a constraint set requiring `typevar` to specialize exactly to `value`."""

    @staticmethod
    def range(
        lower_bound: TypeForm[object],
        typevar: TypeForm[object],
        upper_bound: TypeForm[object],
    ) -> ConstraintSet:
        """
        Returns a constraint set that requires `typevar` to specialize to a type
        that is a supertype of `lower_bound` and a subtype of `upper_bound`.
        """

    @staticmethod
    def always() -> ConstraintSet:
        """Returns a constraint set that is always satisfied"""

    @staticmethod
    def never() -> ConstraintSet:
        """Returns a constraint set that is never satisfied"""

    def implies_subtype_of(self, ty: TypeForm[object], of: TypeForm[object]) -> Self:
        """
        Returns a constraint set that is satisfied when `ty` is a `subtype`_ of
        `of`, assuming that all of the constraints in `self` hold.

        .. _subtype: https://typing.python.org/en/latest/spec/concepts.html#subtype-supertype-and-type-equivalence
        """

    def satisfies(self, other: Self) -> Self:
        """
        Returns whether this constraint set satisfies another — that is, whether
        every specialization that satisfies this constraint set also satisfies
        `other`.
        """

    def exists(self, typevars: TypeForm[tuple[object, ...]]) -> Self:
        """
        Existentially abstracts the given type variables from this constraint set.
        """

    def for_all(self, typevars: TypeForm[tuple[object, ...]]) -> Self:
        """
        Universally abstracts the given type variables from this constraint set.
        """

    def solutions_for(
        self,
        typevar: TypeForm[object],
        *,
        inferable: TypeForm[tuple[object, ...]],
    ) -> tuple[ConstraintSetSolution, ...] | None:
        """
        Returns the explicit solutions inferred for `typevar` across all paths.

        `inferable` specifies all typevars that should be solved for. Every
        solution path is preserved, with its bindings filtered to `typevar`.
        Returns `None` if the constraint set is unsatisfiable.
        """

    def solutions(
        self, *, inferable: TypeForm[tuple[object, ...]]
    ) -> tuple[ConstraintSetSolution, ...] | None:
        """
        Returns all explicit solutions, preserving path and binding order.

        `inferable` specifies all typevars that should be solved for. Each
        solution contains the bindings inferred on one satisfying path.
        Returns `None` if the constraint set is unsatisfiable.
        """

    def __bool__(self) -> bool: ...
    def __eq__(self, other: ConstraintSet) -> bool: ...
    def __ne__(self, other: ConstraintSet) -> bool: ...
    def __and__(self, other: ConstraintSet) -> ConstraintSet: ...
    def __or__(self, other: ConstraintSet) -> ConstraintSet: ...
    def __invert__(self) -> ConstraintSet: ...
    def with_detailed_display(self) -> ConstraintSet:
        """
        Returns a copy of this constraint set that will display the full
        constraint formula when rendered as a string.

        Typically we only display "bool" for a non-trivial constraint set, to
        help ensure that we do not write test cases that depend on how
        constraint sets are rendered. But it can be useful to see the full
        detail for debugging purposes.
        """

class GenericContext:
    """
    The set of typevars that are bound by a generic class, function, or type
    alias.
    """

class Specialization:
    """A mapping of typevars to specific types"""

# -------------------
# Predicates on types
# -------------------

def is_equivalent_to(
    type_a: TypeForm[object], type_b: TypeForm[object]
) -> ConstraintSet:
    """Returns a constraint set that is satisfied when `type_a` and `type_b` are
    `equivalent`_ types.

    .. _equivalent: https://typing.python.org/en/latest/spec/glossary.html#term-equivalent
    """

def is_subtype_of(ty: TypeForm[object], of: TypeForm[object]) -> ConstraintSet:
    """Returns a constraint set that is satisfied when `ty` is a `subtype`_ of `of`.

    .. _subtype: https://typing.python.org/en/latest/spec/concepts.html#subtype-supertype-and-type-equivalence
    """

def is_assignable_to(ty: TypeForm[object], to: TypeForm[object]) -> ConstraintSet:
    """Returns a constraint set that is satisfied when `ty` is `assignable`_ to `to`.

    .. _assignable: https://typing.python.org/en/latest/spec/concepts.html#the-assignable-to-or-consistent-subtyping-relation
    """

def is_constraint_set_assignable_to(
    ty: TypeForm[object],
    to: TypeForm[object],
) -> ConstraintSet:
    """Returns a constraint set that is satisfied when `ty` is `assignable`_ to `to`.

    This differs from `is_assignable_to` in how it treats typevars.
    `is_assignable_to` will assume that all typevars are non-inferable, and will
    require all possible specializations of a typevar to satisfy the relation.
    This method will instead return a constraint set describing which
    specializations (possibly not all of them) satisfy the relation.

    .. _assignable: https://typing.python.org/en/latest/spec/concepts.html#the-assignable-to-or-consistent-subtyping-relation
    """

def is_disjoint_from(
    type_a: TypeForm[object], type_b: TypeForm[object]
) -> ConstraintSet:
    """Returns a constraint set that is satisfied when `type_a` and `type_b` are disjoint types.

    Two types are disjoint if they have no inhabitants in common.
    """

def is_singleton(ty: TypeForm[object]) -> bool:
    """Returns `True` if `ty` is a singleton type with exactly one inhabitant."""

# -------------------
# Operations on types
# -------------------

def generic_context(input: Any) -> GenericContext | None:
    """Returns the generic context of the input (a class, a function, a method, a type alias, ..)
    as a tuple of typevars.

    Returns `None` if the input is not generic.
    """

def into_callable(value: Any) -> Any:
    """Converts a value into a `Callable`, if possible.

    This is the value equivalent of `CallableTypeOf`, which operates on types.
    """

def into_regular_callable(value: Any) -> Any:
    """Converts a value into a regular `Callable`, if possible.

    This is the value equivalent of `RegularCallableTypeOf`, which operates on types.
    """

def dunder_all_names(module: Any) -> tuple[LiteralString, ...] | None:
    """Returns the `__all__` names of a module as a tuple of sorted strings.

    Returns `None` if either the module does not have `__all__` or it has invalid elements.
    """

def enum_members[E: type[Enum]](enum: E) -> tuple[LiteralString, ...]:
    """List all members of an enum."""

def all_members(obj: Any) -> tuple[LiteralString, ...]:
    """Returns a tuple of all members of the given object.

    This nearly emulates `dir(obj)` and `inspect.getmembers(obj)` in Python's
    standard library, but has at least the following differences:

    * `dir` and `inspect.getmembers` may use runtime mutable state to construct
      the list of attributes returned. In contrast, this routine is limited to
      static information only.
    * `dir` will respect an object's `__dir__` implementation, if present, but
      this method (currently) does not.
    """

def has_member(obj: Any, name: LiteralString) -> bool:
    """Returns `True` if the given object has a member with the given name."""

def reveal_protocol_interface(protocol: type) -> None:
    """
    Passing a protocol type to this function will cause ty to emit an info-level
    diagnostic describing the protocol's interface.

    Passing a non-protocol type will cause ty to emit an error diagnostic.
    """

def reveal_mro(cls: type | types.GenericAlias) -> None:
    """Reveal the MRO that ty infers for the given class or generic alias."""

# Special variants of the corresponding protocols in `typing`, but without the
# `__iter__`/`__aiter__` on Iterator/AsyncIterator, which is often omitted in
# practice. These protocols are used for generating better `non-iterable` error
# messages, nothing else.
class Iterator[T](Protocol):
    def __next__(self, /) -> T: ...

class Iterable[T](Protocol):
    def __iter__(self, /) -> Iterator[T]: ...

class AsyncIterator[T](Protocol):
    async def __anext__(self, /) -> T: ...

class AsyncIterable[T](Protocol):
    def __aiter__(self, /) -> AsyncIterator[T]: ...



"""Loading unittests."""

import sys
import unittest.case
import unittest.suite
from collections.abc import Callable, Sequence
from re import Pattern
from types import ModuleType
from typing import Any, Final, TypeAlias
from typing_extensions import deprecated

_SortComparisonMethod: TypeAlias = Callable[[str, str], int]
_SuiteClass: TypeAlias = Callable[[list[unittest.case.TestCase]], unittest.suite.TestSuite]

VALID_MODULE_NAME: Final[Pattern[str]]

class TestLoader:
    """
    This class is responsible for loading tests according to various criteria
    and returning them wrapped in a TestSuite
    """

    errors: list[type[BaseException]]
    testMethodPrefix: str
    sortTestMethodsUsing: _SortComparisonMethod
    testNamePatterns: list[str] | None
    suiteClass: _SuiteClass
    def loadTestsFromTestCase(self, testCaseClass: type[unittest.case.TestCase]) -> unittest.suite.TestSuite:
        """Return a suite of all test cases contained in testCaseClass"""

    if sys.version_info >= (3, 12):
        def loadTestsFromModule(self, module: ModuleType, *, pattern: str | None = None) -> unittest.suite.TestSuite:
            """Return a suite of all test cases contained in the given module"""

    else:
        def loadTestsFromModule(self, module: ModuleType, *args: Any, pattern: str | None = None) -> unittest.suite.TestSuite:
            """Return a suite of all test cases contained in the given module"""

    def loadTestsFromName(self, name: str, module: ModuleType | None = None) -> unittest.suite.TestSuite:
        """Return a suite of all test cases given a string specifier.

        The name may resolve either to a module, a test case class, a
        test method within a test case class, or a callable object which
        returns a TestCase or TestSuite instance.

        The method optionally resolves the names relative to a given module.
        """

    def loadTestsFromNames(self, names: Sequence[str], module: ModuleType | None = None) -> unittest.suite.TestSuite:
        """Return a suite of all test cases found using the given sequence
        of string specifiers. See 'loadTestsFromName()'.
        """

    def getTestCaseNames(self, testCaseClass: type[unittest.case.TestCase]) -> Sequence[str]:
        """Return a sorted sequence of method names found within testCaseClass"""

    def discover(self, start_dir: str, pattern: str = "test*.py", top_level_dir: str | None = None) -> unittest.suite.TestSuite:
        """Find and return all test modules from the specified start
        directory, recursing into subdirectories to find them and return all
        tests found within them. Only test files that match the pattern will
        be loaded. (Using shell style pattern matching.)

        All test modules must be importable from the top level of the project.
        If the start directory is not the top level directory then the top
        level directory must be specified separately.

        If a test package name (directory with '__init__.py') matches the
        pattern then the package will be checked for a 'load_tests' function. If
        this exists then it will be called with (loader, tests, pattern) unless
        the package has already had load_tests called from the same discovery
        invocation, in which case the package module object is not scanned for
        tests - this ensures that when a package uses discover to further
        discover child tests that infinite recursion does not happen.

        If load_tests exists then discovery does *not* recurse into the package,
        load_tests is responsible for loading all tests in the package.

        The pattern is deliberately not stored as a loader attribute so that
        packages can continue discovery themselves. top_level_dir is stored so
        load_tests does not need to pass this argument in to loader.discover().

        Paths are sorted before being imported to ensure reproducible execution
        order even on filesystems with non-alphabetical ordering like ext3/4.
        """

    def _match_path(self, path: str, full_path: str, pattern: str) -> bool: ...

defaultTestLoader: TestLoader

if sys.version_info < (3, 13):
    @deprecated("Deprecated; removed in Python 3.13.")
    def getTestCaseNames(
        testCaseClass: type[unittest.case.TestCase],
        prefix: str,
        sortUsing: _SortComparisonMethod = ...,
        testNamePatterns: list[str] | None = None,
    ) -> Sequence[str]: ...
    @deprecated("Deprecated; removed in Python 3.13.")
    def makeSuite(
        testCaseClass: type[unittest.case.TestCase],
        prefix: str = "test",
        sortUsing: _SortComparisonMethod = ...,
        suiteClass: _SuiteClass = ...,
    ) -> unittest.suite.TestSuite: ...
    @deprecated("Deprecated; removed in Python 3.13.")
    def findTestCases(
        module: ModuleType, prefix: str = "test", sortUsing: _SortComparisonMethod = ..., suiteClass: _SuiteClass = ...
    ) -> unittest.suite.TestSuite: ...



# Tricky sequences of fmt off and on

# Formatted
a +   b

# fmt: off
    # not formatted 1
# fmt: on
a   + b
    # formatted


# fmt: off
    # not formatted 1
# fmt: on
    # not formatted 2
# fmt: off
a   + b
# fmt: on


# fmt: off
    # not formatted 1
# fmt: on
    # formatted 1
# fmt: off
    # not formatted 2
a   + b
# fmt: on
    # formatted
b   + c


# fmt: off
a   + b

    # not formatted
# fmt: on
    # formatted
a   + b


# fmt: off
a   + b

    # not formatted 1
# fmt: on
    # formatted
# fmt: off
    # not formatted 2
a   + b


# fmt: off
a   + b

    # not formatted 1
# fmt: on
    # formatted

# leading
a    + b
# fmt: off

    # leading unformatted
def test  ():
    pass

 # fmt: on

a   + b



"""PlaySound(sound, flags) - play a sound
SND_FILENAME - sound is a wav file name
SND_ALIAS - sound is a registry sound association name
SND_LOOP - Play the sound repeatedly; must also specify SND_ASYNC
SND_MEMORY - sound is a memory image of a wav file
SND_PURGE - stop all instances of the specified sound
SND_ASYNC - PlaySound returns immediately
SND_NODEFAULT - Do not play a default beep if the sound can not be found
SND_NOSTOP - Do not interrupt any sounds currently playing
SND_NOWAIT - Return immediately if the sound driver is busy
SND_APPLICATION - sound is an application-specific alias in the registry.
SND_SENTRY - Triggers a SoundSentry event when the sound is played.
SND_SYNC - Play the sound synchronously, default behavior.
SND_SYSTEM - Assign sound to the audio session for system notification sounds.

Beep(frequency, duration) - Make a beep through the PC speaker.
MessageBeep(type) - Call Windows MessageBeep.
"""

import sys
from _typeshed import ReadableBuffer
from typing import Final, Literal, overload

if sys.platform == "win32":
    SND_APPLICATION: Final = 128
    SND_FILENAME: Final = 131072
    SND_ALIAS: Final = 65536
    SND_LOOP: Final = 8
    SND_MEMORY: Final = 4
    SND_PURGE: Final = 64
    SND_ASYNC: Final = 1
    SND_NODEFAULT: Final = 2
    SND_NOSTOP: Final = 16
    SND_NOWAIT: Final = 8192
    if sys.version_info >= (3, 14):
        SND_SENTRY: Final = 524288
        SND_SYNC: Final = 0
        SND_SYSTEM: Final = 2097152

    MB_ICONASTERISK: Final = 64
    MB_ICONEXCLAMATION: Final = 48
    MB_ICONHAND: Final = 16
    MB_ICONQUESTION: Final = 32
    MB_OK: Final = 0
    if sys.version_info >= (3, 14):
        MB_ICONERROR: Final = 16
        MB_ICONINFORMATION: Final = 64
        MB_ICONSTOP: Final = 16
        MB_ICONWARNING: Final = 48

    def Beep(frequency: int, duration: int) -> None:
        """A wrapper around the Windows Beep API.

        frequency
          Frequency of the sound in hertz.
          Must be in the range 37 through 32,767.
        duration
          How long the sound should play, in milliseconds.
        """

    # Can actually accept anything ORed with 4, and if not it's definitely str, but that's inexpressible
    @overload
    def PlaySound(sound: ReadableBuffer | None, flags: Literal[4]) -> None:
        """A wrapper around the Windows PlaySound API.

        sound
          The sound to play; a filename, data, or None.
        flags
          Flag values, ored together.  See module documentation.
        """
    @overload
    def PlaySound(sound: str | ReadableBuffer | None, flags: int) -> None: ...

    def MessageBeep(type: int = 0) -> None:
        """Call Windows MessageBeep(x).

        x defaults to MB_OK.
        """



from collections.abc import Mapping, Sequence
from types import ModuleType
from typing import Any, Final

__all__ = [
    "_main",
    "freeze_support",
    "set_executable",
    "get_executable",
    "get_preparation_data",
    "get_command_line",
    "import_main_path",
]

WINEXE: Final[bool]
WINSERVICE: Final[bool]

def set_executable(exe: str) -> None: ...
def get_executable() -> str: ...
def is_forking(argv: Sequence[str]) -> bool:
    """
    Return whether commandline indicates we are forking
    """

def freeze_support() -> None:
    """
    Run code for process object if this in not the main process
    """

def get_command_line(**kwds: Any) -> list[str]:
    """
    Returns prefix of command line used for spawning a child process
    """

def spawn_main(pipe_handle: int, parent_pid: int | None = None, tracker_fd: int | None = None) -> None:
    """
    Run code specified by data received over pipe
    """

# undocumented
def _main(fd: int, parent_sentinel: int) -> int: ...
def get_preparation_data(name: str) -> dict[str, Any]:
    """
    Return info about parent needed by child to unpickle process object
    """

old_main_modules: list[ModuleType]

def prepare(data: Mapping[str, Any]) -> None:
    """
    Try to get current process ready to unpickle process object
    """

def import_main_path(main_path: str) -> None:
    """
    Set sys.modules['__main__'] to module at main_path
    """



"""The functions in this module allow compression and decompression using the
zlib library, which is based on GNU zip.

adler32(string[, start]) -- Compute an Adler-32 checksum.
adler32_combine(adler1, adler2, len2, /) -- Combine two Adler-32 checksums.
compress(data[, level]) -- Compress data, with compression level 0-9 or -1.
compressobj([level[, ...]]) -- Return a compressor object.
crc32(string[, start]) -- Compute a CRC-32 checksum.
crc32_combine(crc1, crc2, len2, /) -- Combine two CRC-32 checksums.
decompress(string,[wbits],[bufsize]) -- Decompresses a compressed string.
decompressobj([wbits[, zdict]]) -- Return a decompressor object.

'wbits' is window buffer size and container format.
Compressor objects support compress() and flush() methods; decompressor
objects support decompress() and flush().
"""

import sys
from _typeshed import ReadableBuffer
from typing import Any, Final, final, type_check_only
from typing_extensions import Self

DEFLATED: Final = 8
DEF_MEM_LEVEL: Final[int]
DEF_BUF_SIZE: Final = 16384
MAX_WBITS: Final[int]
ZLIB_VERSION: Final[str]
ZLIB_RUNTIME_VERSION: Final[str]
Z_NO_COMPRESSION: Final = 0
Z_PARTIAL_FLUSH: Final = 1
Z_BEST_COMPRESSION: Final = 9
Z_BEST_SPEED: Final = 1
Z_BLOCK: Final = 5
Z_DEFAULT_COMPRESSION: Final = -1
Z_DEFAULT_STRATEGY: Final = 0
Z_FILTERED: Final = 1
Z_FINISH: Final = 4
Z_FIXED: Final = 4
Z_FULL_FLUSH: Final = 3
Z_HUFFMAN_ONLY: Final = 2
Z_NO_FLUSH: Final = 0
Z_RLE: Final = 3
Z_SYNC_FLUSH: Final = 2
Z_TREES: Final = 6

if sys.version_info >= (3, 14):
    # Available when zlib was built with zlib-ng
    ZLIBNG_VERSION: Final[str]

class error(Exception): ...

# This class is not exposed at runtime. It calls itself zlib.Compress.
@final
@type_check_only
class _Compress:
    def __copy__(self) -> Self: ...
    def __deepcopy__(self, memo: Any, /) -> Self: ...
    def compress(self, data: ReadableBuffer, /) -> bytes: ...
    def flush(self, mode: int = 4, /) -> bytes: ...
    def copy(self) -> _Compress: ...

# This class is not exposed at runtime. It calls itself zlib.Decompress.
@final
@type_check_only
class _Decompress:
    @property
    def unused_data(self) -> bytes: ...
    @property
    def unconsumed_tail(self) -> bytes: ...
    @property
    def eof(self) -> bool: ...
    def __copy__(self) -> Self: ...
    def __deepcopy__(self, memo: Any, /) -> Self: ...
    def decompress(self, data: ReadableBuffer, /, max_length: int = 0) -> bytes: ...
    def flush(self, length: int = 16384, /) -> bytes: ...
    def copy(self) -> _Decompress: ...

def adler32(data: ReadableBuffer, value: int = 1, /) -> int:
    """Compute an Adler-32 checksum of data.

      value
        Starting value of the checksum.

    The returned checksum is an integer.
    """

if sys.version_info >= (3, 15):
    def adler32_combine(adler1: int, adler2: int, len2: int, /) -> int:
        """Combine two Adler-32 checksums into one.

          adler1
            Adler-32 checksum for sequence A
          adler2
            Adler-32 checksum for sequence B
          len2
            Length of sequence B

        Given the Adler-32 checksum 'adler1' of a sequence A and the
        Adler-32 checksum 'adler2' of a sequence B of length 'len2',
        return the Adler-32 checksum of A and B concatenated.
        """

if sys.version_info >= (3, 11):
    def compress(data: ReadableBuffer, /, level: int = -1, wbits: int = 15) -> bytes:
        """Returns a bytes object containing compressed data.

        data
          Binary data to be compressed.
        level
          Compression level, in 0-9 or -1.
        wbits
          The window buffer size and container format.
        """

else:
    def compress(data: ReadableBuffer, /, level: int = -1) -> bytes:
        """Returns a bytes object containing compressed data.

        data
          Binary data to be compressed.
        level
          Compression level, in 0-9 or -1.
        """

def compressobj(
    level: int = -1, method: int = 8, wbits: int = 15, memLevel: int = 8, strategy: int = 0, zdict: ReadableBuffer | None = None
) -> _Compress:
    """Return a compressor object.

    level
      The compression level (an integer in the range 0-9 or -1; default is
      currently equivalent to 6).  Higher compression levels are slower,
      but produce smaller results.
    method
      The compression algorithm.  If given, this must be DEFLATED.
    wbits
      +9 to +15: The base-two logarithm of the window size.  Include a zlib
          container.
      -9 to -15: Generate a raw stream.
      +25 to +31: Include a gzip container.
    memLevel
      Controls the amount of memory used for internal compression state.
      Valid values range from 1 to 9.  Higher values result in higher memory
      usage, faster compression, and smaller output.
    strategy
      Used to tune the compression algorithm.  Possible values are
      Z_DEFAULT_STRATEGY, Z_FILTERED, and Z_HUFFMAN_ONLY.
    zdict
      The predefined compression dictionary - a sequence of bytes
      containing subsequences that are likely to occur in the input data.
    """

def crc32(data: ReadableBuffer, value: int = 0, /) -> int:
    """Compute a CRC-32 checksum of data.

      value
        Starting value of the checksum.

    The returned checksum is an integer.
    """

if sys.version_info >= (3, 15):
    def crc32_combine(crc1: int, crc2: int, len2: int, /) -> int:
        """Combine two CRC-32 checksums into one.

          crc1
            CRC-32 checksum for sequence A
          crc2
            CRC-32 checksum for sequence B
          len2
            Length of sequence B

        Given the CRC-32 checksum 'crc1' of a sequence A and the
        CRC-32 checksum 'crc2' of a sequence B of length 'len2',
        return the CRC-32 checksum of A and B concatenated.
        """

def decompress(data: ReadableBuffer, /, wbits: int = 15, bufsize: int = 16384) -> bytes:
    """Returns a bytes object containing the uncompressed data.

    data
      Compressed data.
    wbits
      The window buffer size and container format.
    bufsize
      The initial output buffer size.
    """

def decompressobj(wbits: int = 15, zdict: ReadableBuffer = b"") -> _Decompress:
    """Return a decompressor object.

    wbits
      The window buffer size and container format.
    zdict
      The predefined compression dictionary.  This must be the same
      dictionary as used by the compressor that produced the input data.
    """



""" "Executable documentation" for the pickle module.

Extensive comments about the pickle protocols and pickle-machine opcodes
can be found here.  Some functions meant for external use:

genops(pickle)
   Generate all the opcodes in a pickle, as (opcode, arg, position) triples.

dis(pickle, out=None, memo=None, indentlevel=4)
   Print a symbolic disassembly of a pickle.
"""

import sys
from collections.abc import Callable, Iterator, MutableMapping
from typing import IO, Any, Final, TypeAlias

__all__ = ["dis", "genops", "optimize"]

_Reader: TypeAlias = Callable[[IO[bytes]], Any]
bytes_types: tuple[type[Any], ...]

UP_TO_NEWLINE: Final = -1
TAKEN_FROM_ARGUMENT1: Final = -2
TAKEN_FROM_ARGUMENT4: Final = -3
TAKEN_FROM_ARGUMENT4U: Final = -4
TAKEN_FROM_ARGUMENT8U: Final = -5

class ArgumentDescriptor:
    __slots__ = ("name", "n", "reader", "doc")
    name: str
    n: int
    reader: _Reader
    doc: str
    def __init__(self, name: str, n: int, reader: _Reader, doc: str) -> None: ...

def read_uint1(f: IO[bytes]) -> int:
    """
    >>> import io
    >>> read_uint1(io.BytesIO(b'\\xff'))
    255
    """

uint1: ArgumentDescriptor

def read_uint2(f: IO[bytes]) -> int:
    """
    >>> import io
    >>> read_uint2(io.BytesIO(b'\\xff\\x00'))
    255
    >>> read_uint2(io.BytesIO(b'\\xff\\xff'))
    65535
    """

uint2: ArgumentDescriptor

def read_int4(f: IO[bytes]) -> int:
    """
    >>> import io
    >>> read_int4(io.BytesIO(b'\\xff\\x00\\x00\\x00'))
    255
    >>> read_int4(io.BytesIO(b'\\x00\\x00\\x00\\x80')) == -(2**31)
    True
    """

int4: ArgumentDescriptor

def read_uint4(f: IO[bytes]) -> int:
    """
    >>> import io
    >>> read_uint4(io.BytesIO(b'\\xff\\x00\\x00\\x00'))
    255
    >>> read_uint4(io.BytesIO(b'\\x00\\x00\\x00\\x80')) == 2**31
    True
    """

uint4: ArgumentDescriptor

def read_uint8(f: IO[bytes]) -> int:
    """
    >>> import io
    >>> read_uint8(io.BytesIO(b'\\xff\\x00\\x00\\x00\\x00\\x00\\x00\\x00'))
    255
    >>> read_uint8(io.BytesIO(b'\\xff' * 8)) == 2**64-1
    True
    """

uint8: ArgumentDescriptor

if sys.version_info >= (3, 12):
    def read_stringnl(f: IO[bytes], decode: bool = True, stripquotes: bool = True, *, encoding: str = "latin-1") -> bytes | str:
        """
        >>> import io
        >>> read_stringnl(io.BytesIO(b"'abcd'\\nefg\\n"))
        'abcd'

        >>> read_stringnl(io.BytesIO(b"\\n"))
        Traceback (most recent call last):
        ...
        ValueError: no string quotes around b''

        >>> read_stringnl(io.BytesIO(b"\\n"), stripquotes=False)
        ''

        >>> read_stringnl(io.BytesIO(b"''\\n"))
        ''

        >>> read_stringnl(io.BytesIO(b'"abcd"'))
        Traceback (most recent call last):
        ...
        ValueError: no newline found when trying to read stringnl

        Embedded escapes are undone in the result.
        >>> read_stringnl(io.BytesIO(br"'a\\n\\\\b\\x00c\\td'" + b"\\n'e'"))
        'a\\n\\\\b\\x00c\\td'
        """

else:
    def read_stringnl(f: IO[bytes], decode: bool = True, stripquotes: bool = True) -> bytes | str:
        """
        >>> import io
        >>> read_stringnl(io.BytesIO(b"'abcd'\\nefg\\n"))
        'abcd'

        >>> read_stringnl(io.BytesIO(b"\\n"))
        Traceback (most recent call last):
        ...
        ValueError: no string quotes around b''

        >>> read_stringnl(io.BytesIO(b"\\n"), stripquotes=False)
        ''

        >>> read_stringnl(io.BytesIO(b"''\\n"))
        ''

        >>> read_stringnl(io.BytesIO(b'"abcd"'))
        Traceback (most recent call last):
        ...
        ValueError: no newline found when trying to read stringnl

        Embedded escapes are undone in the result.
        >>> read_stringnl(io.BytesIO(br"'a\\n\\\\b\\x00c\\td'" + b"\\n'e'"))
        'a\\n\\\\b\\x00c\\td'
        """

stringnl: ArgumentDescriptor

def read_stringnl_noescape(f: IO[bytes]) -> str: ...

stringnl_noescape: ArgumentDescriptor

def read_stringnl_noescape_pair(f: IO[bytes]) -> str:
    """
    >>> import io
    >>> read_stringnl_noescape_pair(io.BytesIO(b"Queue\\nEmpty\\njunk"))
    'Queue Empty'
    """

stringnl_noescape_pair: ArgumentDescriptor

def read_string1(f: IO[bytes]) -> str:
    """
    >>> import io
    >>> read_string1(io.BytesIO(b"\\x00"))
    ''
    >>> read_string1(io.BytesIO(b"\\x03abcdef"))
    'abc'
    """

string1: ArgumentDescriptor

def read_string4(f: IO[bytes]) -> str:
    """
    >>> import io
    >>> read_string4(io.BytesIO(b"\\x00\\x00\\x00\\x00abc"))
    ''
    >>> read_string4(io.BytesIO(b"\\x03\\x00\\x00\\x00abcdef"))
    'abc'
    >>> read_string4(io.BytesIO(b"\\x00\\x00\\x00\\x03abcdef"))
    Traceback (most recent call last):
    ...
    ValueError: expected 50331648 bytes in a string4, but only 6 remain
    """

string4: ArgumentDescriptor

def read_bytes1(f: IO[bytes]) -> bytes:
    """
    >>> import io
    >>> read_bytes1(io.BytesIO(b"\\x00"))
    b''
    >>> read_bytes1(io.BytesIO(b"\\x03abcdef"))
    b'abc'
    """

bytes1: ArgumentDescriptor

def read_bytes4(f: IO[bytes]) -> bytes:
    """
    >>> import io
    >>> read_bytes4(io.BytesIO(b"\\x00\\x00\\x00\\x00abc"))
    b''
    >>> read_bytes4(io.BytesIO(b"\\x03\\x00\\x00\\x00abcdef"))
    b'abc'
    >>> read_bytes4(io.BytesIO(b"\\x00\\x00\\x00\\x03abcdef"))
    Traceback (most recent call last):
    ...
    ValueError: expected 50331648 bytes in a bytes4, but only 6 remain
    """

bytes4: ArgumentDescriptor

def read_bytes8(f: IO[bytes]) -> bytes:
    """
    >>> import io, struct, sys
    >>> read_bytes8(io.BytesIO(b"\\x00\\x00\\x00\\x00\\x00\\x00\\x00\\x00abc"))
    b''
    >>> read_bytes8(io.BytesIO(b"\\x03\\x00\\x00\\x00\\x00\\x00\\x00\\x00abcdef"))
    b'abc'
    >>> bigsize8 = struct.pack("<Q", sys.maxsize//3)
    >>> read_bytes8(io.BytesIO(bigsize8 + b"abcdef"))  #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ...
    ValueError: expected ... bytes in a bytes8, but only 6 remain
    """

bytes8: ArgumentDescriptor

def read_unicodestringnl(f: IO[bytes]) -> str:
    """
    >>> import io
    >>> read_unicodestringnl(io.BytesIO(b"abc\\\\uabcd\\njunk")) == 'abc\\uabcd'
    True
    """

unicodestringnl: ArgumentDescriptor

def read_unicodestring1(f: IO[bytes]) -> str:
    """
    >>> import io
    >>> s = 'abcd\\uabcd'
    >>> enc = s.encode('utf-8')
    >>> enc
    b'abcd\\xea\\xaf\\x8d'
    >>> n = bytes([len(enc)])  # little-endian 1-byte length
    >>> t = read_unicodestring1(io.BytesIO(n + enc + b'junk'))
    >>> s == t
    True

    >>> read_unicodestring1(io.BytesIO(n + enc[:-1]))
    Traceback (most recent call last):
    ...
    ValueError: expected 7 bytes in a unicodestring1, but only 6 remain
    """

unicodestring1: ArgumentDescriptor

def read_unicodestring4(f: IO[bytes]) -> str:
    """
    >>> import io
    >>> s = 'abcd\\uabcd'
    >>> enc = s.encode('utf-8')
    >>> enc
    b'abcd\\xea\\xaf\\x8d'
    >>> n = bytes([len(enc), 0, 0, 0])  # little-endian 4-byte length
    >>> t = read_unicodestring4(io.BytesIO(n + enc + b'junk'))
    >>> s == t
    True

    >>> read_unicodestring4(io.BytesIO(n + enc[:-1]))
    Traceback (most recent call last):
    ...
    ValueError: expected 7 bytes in a unicodestring4, but only 6 remain
    """

unicodestring4: ArgumentDescriptor

def read_unicodestring8(f: IO[bytes]) -> str:
    """
    >>> import io
    >>> s = 'abcd\\uabcd'
    >>> enc = s.encode('utf-8')
    >>> enc
    b'abcd\\xea\\xaf\\x8d'
    >>> n = bytes([len(enc)]) + b'\\0' * 7  # little-endian 8-byte length
    >>> t = read_unicodestring8(io.BytesIO(n + enc + b'junk'))
    >>> s == t
    True

    >>> read_unicodestring8(io.BytesIO(n + enc[:-1]))
    Traceback (most recent call last):
    ...
    ValueError: expected 7 bytes in a unicodestring8, but only 6 remain
    """

unicodestring8: ArgumentDescriptor

def read_decimalnl_short(f: IO[bytes]) -> int:
    """
    >>> import io
    >>> read_decimalnl_short(io.BytesIO(b"1234\\n56"))
    1234

    >>> read_decimalnl_short(io.BytesIO(b"1234L\\n56"))
    Traceback (most recent call last):
    ...
    ValueError: invalid literal for int() with base 10: b'1234L'
    """

def read_decimalnl_long(f: IO[bytes]) -> int:
    """
    >>> import io

    >>> read_decimalnl_long(io.BytesIO(b"1234L\\n56"))
    1234

    >>> read_decimalnl_long(io.BytesIO(b"123456789012345678901234L\\n6"))
    123456789012345678901234
    """

decimalnl_short: ArgumentDescriptor
decimalnl_long: ArgumentDescriptor

def read_floatnl(f: IO[bytes]) -> float:
    """
    >>> import io
    >>> read_floatnl(io.BytesIO(b"-1.25\\n6"))
    -1.25
    """

floatnl: ArgumentDescriptor

def read_float8(f: IO[bytes]) -> float:
    """
    >>> import io, struct
    >>> raw = struct.pack(">d", -1.25)
    >>> raw
    b'\\xbf\\xf4\\x00\\x00\\x00\\x00\\x00\\x00'
    >>> read_float8(io.BytesIO(raw + b"\\n"))
    -1.25
    """

float8: ArgumentDescriptor

def read_long1(f: IO[bytes]) -> int:
    """
    >>> import io
    >>> read_long1(io.BytesIO(b"\\x00"))
    0
    >>> read_long1(io.BytesIO(b"\\x02\\xff\\x00"))
    255
    >>> read_long1(io.BytesIO(b"\\x02\\xff\\x7f"))
    32767
    >>> read_long1(io.BytesIO(b"\\x02\\x00\\xff"))
    -256
    >>> read_long1(io.BytesIO(b"\\x02\\x00\\x80"))
    -32768
    """

long1: ArgumentDescriptor

def read_long4(f: IO[bytes]) -> int:
    """
    >>> import io
    >>> read_long4(io.BytesIO(b"\\x02\\x00\\x00\\x00\\xff\\x00"))
    255
    >>> read_long4(io.BytesIO(b"\\x02\\x00\\x00\\x00\\xff\\x7f"))
    32767
    >>> read_long4(io.BytesIO(b"\\x02\\x00\\x00\\x00\\x00\\xff"))
    -256
    >>> read_long4(io.BytesIO(b"\\x02\\x00\\x00\\x00\\x00\\x80"))
    -32768
    >>> read_long1(io.BytesIO(b"\\x00\\x00\\x00\\x00"))
    0
    """

long4: ArgumentDescriptor

class StackObject:
    __slots__ = ("name", "obtype", "doc")
    name: str
    obtype: type[Any] | tuple[type[Any], ...]
    doc: str
    def __init__(self, name: str, obtype: type[Any] | tuple[type[Any], ...], doc: str) -> None: ...

pyint: StackObject
pylong: StackObject
pyinteger_or_bool: StackObject
pybool: StackObject
pyfloat: StackObject
pybytes_or_str: StackObject
pystring: StackObject
pybytes: StackObject
pyunicode: StackObject
pynone: StackObject
pytuple: StackObject
pylist: StackObject
pydict: StackObject
pyset: StackObject
pyfrozenset: StackObject
anyobject: StackObject
markobject: StackObject
stackslice: StackObject

class OpcodeInfo:
    __slots__ = ("name", "code", "arg", "stack_before", "stack_after", "proto", "doc")
    name: str
    code: str
    arg: ArgumentDescriptor | None
    stack_before: list[StackObject]
    stack_after: list[StackObject]
    proto: int
    doc: str
    def __init__(
        self,
        name: str,
        code: str,
        arg: ArgumentDescriptor | None,
        stack_before: list[StackObject],
        stack_after: list[StackObject],
        proto: int,
        doc: str,
    ) -> None: ...

opcodes: list[OpcodeInfo]

def genops(pickle: bytes | bytearray | IO[bytes]) -> Iterator[tuple[OpcodeInfo, Any | None, int | None]]:
    """Generate all the opcodes in a pickle.

    'pickle' is a file-like object, or string, containing the pickle.

    Each opcode in the pickle is generated, from the current pickle position,
    stopping after a STOP opcode is delivered.  A triple is generated for
    each opcode:

        opcode, arg, pos

    opcode is an OpcodeInfo record, describing the current opcode.

    If the opcode has an argument embedded in the pickle, arg is its decoded
    value, as a Python object.  If the opcode doesn't have an argument, arg
    is None.

    If the pickle has a tell() method, pos was the value of pickle.tell()
    before reading the current opcode.  If the pickle is a bytes object,
    it's wrapped in a BytesIO object, and the latter's tell() result is
    used.  Else (the pickle doesn't have a tell(), and it's not obvious how
    to query its current position) pos is None.
    """

def optimize(p: bytes | bytearray | IO[bytes]) -> bytes:
    """Optimize a pickle string by removing unused PUT opcodes"""

def dis(
    pickle: bytes | bytearray | IO[bytes],
    out: IO[str] | None = None,
    memo: MutableMapping[int, Any] | None = None,
    indentlevel: int = 4,
    annotate: int = 0,
) -> None:
    """Produce a symbolic disassembly of a pickle.

    'pickle' is a file-like object, or string, containing a (at least one)
    pickle.  The pickle is disassembled from the current position, through
    the first STOP opcode encountered.

    Optional arg 'out' is a file-like object to which the disassembly is
    printed.  It defaults to sys.stdout.

    Optional arg 'memo' is a Python dict, used as the pickle's memo.  It
    may be mutated by dis(), if the pickle contains PUT or BINPUT opcodes.
    Passing the same memo object to another dis() call then allows disassembly
    to proceed across multiple pickles that were all created by the same
    pickler with the same memo.  Ordinarily you don't need to worry about this.

    Optional arg 'indentlevel' is the number of blanks by which to indent
    a new MARK level.  It defaults to 4.

    Optional arg 'annotate' if nonzero instructs dis() to add short
    description of the opcode on each line of disassembled output.
    The value given to 'annotate' must be an integer and is used as a
    hint for the column where annotation should start.  The default
    value is 0, meaning no annotations.

    In addition to printing the disassembly, some sanity checks are made:

    + All embedded opcode arguments "make sense".

    + Explicit and implicit pop operations have enough items on the stack.

    + When an opcode implicitly refers to a markobject, a markobject is
      actually on the stack.

    + A memo entry isn't referenced before it's defined.

    + The markobject isn't stored in the memo.
    """



"""A ScrolledText widget feels like a text widget but also has a
vertical scroll bar on its right.  (Later, options may be added to
add a horizontal bar as well, to make the bars disappear
automatically when not needed, to move them to the other side of the
window, etc.)

Configuration options are passed to the Text widget.
A Frame widget is inserted between the master and the text, to hold
the Scrollbar widget.
Most methods calls are inherited from the Text widget; Pack, Grid and
Place methods are redirected to the Frame widget however.
"""

from tkinter import Frame, Misc, Scrollbar, Text

__all__ = ["ScrolledText"]

# The methods from Pack, Place, and Grid are dynamically added over the parent's impls
class ScrolledText(Text):
    frame: Frame
    vbar: Scrollbar
    def __init__(self, master: Misc | None = None, **kwargs) -> None: ...



"""Random variable generators.

    bytes
    -----
           uniform bytes (values between 0 and 255)

    integers
    --------
           uniform within range

    sequences
    ---------
           pick random element
           pick random sample
           pick weighted random sample
           generate random permutation

    distributions on the real line:
    ------------------------------
           uniform
           triangular
           normal (Gaussian)
           lognormal
           negative exponential
           gamma
           beta
           pareto
           Weibull

    distributions on the circle (angles 0 to 2pi)
    ---------------------------------------------
           circular uniform
           von Mises

    discrete distributions
    ----------------------
           binomial


General notes on the underlying Mersenne Twister core generator:

* The period is 2**19937-1.
* It is one of the most extensively tested generators in existence.
* The random() method is implemented in C, executes in a single Python step,
  and is, therefore, threadsafe.

"""

import _random
import sys
from _typeshed import SupportsLenAndGetItem
from collections.abc import Callable, Iterable, MutableSequence, Sequence, Set as AbstractSet
from fractions import Fraction
from typing import Any, ClassVar, TypeVar, overload
from typing_extensions import Never, deprecated

__all__ = [
    "Random",
    "seed",
    "random",
    "uniform",
    "randint",
    "choice",
    "sample",
    "randrange",
    "shuffle",
    "normalvariate",
    "lognormvariate",
    "expovariate",
    "vonmisesvariate",
    "gammavariate",
    "triangular",
    "gauss",
    "betavariate",
    "paretovariate",
    "weibullvariate",
    "getstate",
    "setstate",
    "getrandbits",
    "choices",
    "SystemRandom",
    "randbytes",
]

if sys.version_info >= (3, 12):
    __all__ += ["binomialvariate"]

_T = TypeVar("_T")

class Random(_random.Random):
    """Random number generator base class used by bound module functions.

    Used to instantiate instances of Random to get generators that don't
    share state.

    Class Random can also be subclassed if you want to use a different basic
    generator of your own devising: in that case, override the following
    methods:  random(), seed(), getstate(), and setstate().
    Optionally, implement a getrandbits() method so that randrange()
    can cover arbitrarily large ranges.

    """

    VERSION: ClassVar[int]
    def __init__(self, x: int | float | str | bytes | bytearray | None = None) -> None:  # noqa: Y041
        """Initialize an instance.

        Optional argument x controls seeding, as for Random.seed().
        """

    # Using other `seed` types is deprecated since 3.9 and removed in 3.11
    # Ignore Y041, since random.seed doesn't treat int like a float subtype. Having an explicit
    # int better documents conventional usage of random.seed.

    def seed(self, a: int | float | str | bytes | bytearray | None = None, version: int = 2) -> None:  # type: ignore[override]  # noqa: Y041
        """Initialize internal state from a seed.

        The only supported seed types are None, int, float,
        str, bytes, and bytearray.

        None or no argument seeds from current time or from an operating
        system specific randomness source if available.

        If *a* is an int, all bits are used.

        For version 2 (the default), all of the bits are used if *a* is a str,
        bytes, or bytearray.  For version 1 (provided for reproducing random
        sequences from older versions of Python), the algorithm for str and
        bytes generates a narrower range of seeds.

        """

    def getstate(self) -> tuple[Any, ...]:
        """Return internal state; can be passed to setstate() later."""

    def setstate(self, state: tuple[Any, ...]) -> None:
        """Restore internal state from object returned by getstate()."""

    def randrange(self, start: int, stop: int | None = None, step: int = 1) -> int:
        """Choose a random item from range(stop) or range(start, stop[, step]).

        Roughly equivalent to ``choice(range(start, stop, step))`` but
        supports arbitrarily large ranges and is optimized for common cases.

        """

    def randint(self, a: int, b: int) -> int:
        """Return random integer in range [a, b], including both end points."""

    def randbytes(self, n: int) -> bytes:
        """Generate n random bytes."""

    def choice(self, seq: SupportsLenAndGetItem[_T]) -> _T:
        """Choose a random element from a non-empty sequence."""

    def choices(
        self,
        population: SupportsLenAndGetItem[_T],
        weights: Sequence[float | Fraction] | None = None,
        *,
        cum_weights: Sequence[float | Fraction] | None = None,
        k: int = 1,
    ) -> list[_T]:
        """Return a k sized list of population elements chosen with replacement.

        If the relative weights or cumulative weights are not specified,
        the selections are made with equal probability.

        """

    if sys.version_info >= (3, 11):
        def shuffle(self, x: MutableSequence[Any]) -> None:
            """Shuffle list x in place, and return None."""

    else:
        @overload
        def shuffle(self, x: MutableSequence[Any]) -> None:
            """Shuffle list x in place, and return None.

            Optional argument random is a 0-argument function returning a
            random float in [0.0, 1.0); if it is the default None, the
            standard random.random will be used.

            """
        @overload
        @deprecated("The `random` parameter is deprecated since Python 3.9; removed in Python 3.11.")
        def shuffle(self, x: MutableSequence[Any], random: Callable[[], float] | None = None) -> None: ...

    if sys.version_info >= (3, 11):
        def sample(self, population: Sequence[_T], k: int, *, counts: Iterable[int] | None = None) -> list[_T]:
            """Chooses k unique random elements from a population sequence.

            Returns a new list containing elements from the population while
            leaving the original population unchanged.  The resulting list is
            in selection order so that all sub-slices will also be valid random
            samples.  This allows raffle winners (the sample) to be partitioned
            into grand prize and second place winners (the subslices).

            Members of the population need not be hashable or unique.  If the
            population contains repeats, then each occurrence is a possible
            selection in the sample.

            Repeated elements can be specified one at a time or with the optional
            counts parameter.  For example:

                sample(['red', 'blue'], counts=[4, 2], k=5)

            is equivalent to:

                sample(['red', 'red', 'red', 'red', 'blue', 'blue'], k=5)

            To choose a sample from a range of integers, use range() for the
            population argument.  This is especially fast and space efficient
            for sampling from a large population:

                sample(range(10000000), 60)

            """

    else:
        def sample(self, population: Sequence[_T] | AbstractSet[_T], k: int, *, counts: Iterable[int] | None = None) -> list[_T]:
            """Chooses k unique random elements from a population sequence or set.

            Returns a new list containing elements from the population while
            leaving the original population unchanged.  The resulting list is
            in selection order so that all sub-slices will also be valid random
            samples.  This allows raffle winners (the sample) to be partitioned
            into grand prize and second place winners (the subslices).

            Members of the population need not be hashable or unique.  If the
            population contains repeats, then each occurrence is a possible
            selection in the sample.

            Repeated elements can be specified one at a time or with the optional
            counts parameter.  For example:

                sample(['red', 'blue'], counts=[4, 2], k=5)

            is equivalent to:

                sample(['red', 'red', 'red', 'red', 'blue', 'blue'], k=5)

            To choose a sample from a range of integers, use range() for the
            population argument.  This is especially fast and space efficient
            for sampling from a large population:

                sample(range(10000000), 60)

            """

    def uniform(self, a: float, b: float) -> float:
        """Get a random number in the range [a, b) or [a, b] depending on rounding.

        The mean (expected value) and variance of the random variable are:

            E[X] = (a + b) / 2
            Var[X] = (b - a) ** 2 / 12

        """

    def triangular(self, low: float = 0.0, high: float = 1.0, mode: float | None = None) -> float:
        """Triangular distribution.

        Continuous distribution bounded by given lower and upper limits,
        and having a given mode value in-between.

        http://en.wikipedia.org/wiki/Triangular_distribution

        The mean (expected value) and variance of the random variable are:

            E[X] = (low + high + mode) / 3
            Var[X] = (low**2 + high**2 + mode**2 - low*high - low*mode - high*mode) / 18

        """

    if sys.version_info >= (3, 12):
        def binomialvariate(self, n: int = 1, p: float = 0.5) -> int:
            """Binomial random variable.

            Gives the number of successes for *n* independent trials
            with the probability of success in each trial being *p*:

                sum(random() < p for i in range(n))

            Returns an integer in the range:

                0 <= X <= n

            The integer is chosen with the probability:

                P(X == k) = math.comb(n, k) * p ** k * (1 - p) ** (n - k)

            The mean (expected value) and variance of the random variable are:

                E[X] = n * p
                Var[X] = n * p * (1 - p)

            """

    def betavariate(self, alpha: float, beta: float) -> float:
        """Beta distribution.

        Conditions on the parameters are alpha > 0 and beta > 0.
        Returned values range between 0 and 1.

        The mean (expected value) and variance of the random variable are:

            E[X] = alpha / (alpha + beta)
            Var[X] = alpha * beta / ((alpha + beta)**2 * (alpha + beta + 1))

        """

    if sys.version_info >= (3, 12):
        def expovariate(self, lambd: float = 1.0) -> float:
            """Exponential distribution.

            lambd is 1.0 divided by the desired mean.  It should be
            nonzero.  (The parameter would be called "lambda", but that is
            a reserved word in Python.)  Returned values range from 0 to
            positive infinity if lambd is positive, and from negative
            infinity to 0 if lambd is negative.

            The mean (expected value) and variance of the random variable are:

                E[X] = 1 / lambd
                Var[X] = 1 / lambd ** 2

            """

    else:
        def expovariate(self, lambd: float) -> float:
            """Exponential distribution.

            lambd is 1.0 divided by the desired mean.  It should be
            nonzero.  (The parameter would be called "lambda", but that is
            a reserved word in Python.)  Returned values range from 0 to
            positive infinity if lambd is positive, and from negative
            infinity to 0 if lambd is negative.

            """

    def gammavariate(self, alpha: float, beta: float) -> float:
        """Gamma distribution.  Not the gamma function!

        Conditions on the parameters are alpha > 0 and beta > 0.

        The probability distribution function is:

                    x ** (alpha - 1) * math.exp(-x / beta)
          pdf(x) =  --------------------------------------
                      math.gamma(alpha) * beta ** alpha

        The mean (expected value) and variance of the random variable are:

            E[X] = alpha * beta
            Var[X] = alpha * beta ** 2

        """

    if sys.version_info >= (3, 11):
        def gauss(self, mu: float = 0.0, sigma: float = 1.0) -> float:
            """Gaussian distribution.

            mu is the mean, and sigma is the standard deviation.  This is
            slightly faster than the normalvariate() function.

            Not thread-safe without a lock around calls.

            """

        def normalvariate(self, mu: float = 0.0, sigma: float = 1.0) -> float:
            """Normal distribution.

            mu is the mean, and sigma is the standard deviation.

            """

    else:
        def gauss(self, mu: float, sigma: float) -> float:
            """Gaussian distribution.

            mu is the mean, and sigma is the standard deviation.  This is
            slightly faster than the normalvariate() function.

            Not thread-safe without a lock around calls.

            """

        def normalvariate(self, mu: float, sigma: float) -> float:
            """Normal distribution.

            mu is the mean, and sigma is the standard deviation.

            """

    def lognormvariate(self, mu: float, sigma: float) -> float:
        """Log normal distribution.

        If you take the natural logarithm of this distribution, you'll get a
        normal distribution with mean mu and standard deviation sigma.
        mu can have any value, and sigma must be greater than zero.

        """

    def vonmisesvariate(self, mu: float, kappa: float) -> float:
        """Circular data distribution.

        mu is the mean angle, expressed in radians between 0 and 2*pi, and
        kappa is the concentration parameter, which must be greater than or
        equal to zero.  If kappa is equal to zero, this distribution reduces
        to a uniform random angle over the range 0 to 2*pi.

        """

    def paretovariate(self, alpha: float) -> float:
        """Pareto distribution.  alpha is the shape parameter."""

    def weibullvariate(self, alpha: float, beta: float) -> float:
        """Weibull distribution.

        alpha is the scale parameter and beta is the shape parameter.

        """

# SystemRandom is not implemented for all OS's; good on Windows & Linux
class SystemRandom(Random):
    """Alternate random number generator using sources provided
    by the operating system (such as /dev/urandom on Unix or
    CryptGenRandom on Windows).

     Not available on all systems (see os.urandom() for details).

    """

    def getrandbits(self, k: int) -> int:  # k can be passed by keyword
        """getrandbits(k) -> x.  Generates an int with k random bits."""

    def getstate(self, *args: Any, **kwds: Any) -> Never:
        """Method should not be called for a system random number generator."""

    def setstate(self, *args: Any, **kwds: Any) -> Never:
        """Method should not be called for a system random number generator."""

_inst: Random
seed = _inst.seed
"""Initialize internal state from a seed.

The only supported seed types are None, int, float,
str, bytes, and bytearray.

None or no argument seeds from current time or from an operating
system specific randomness source if available.

If *a* is an int, all bits are used.

For version 2 (the default), all of the bits are used if *a* is a str,
bytes, or bytearray.  For version 1 (provided for reproducing random
sequences from older versions of Python), the algorithm for str and
bytes generates a narrower range of seeds.

"""

random = _inst.random
uniform = _inst.uniform
"""Get a random number in the range [a, b) or [a, b] depending on rounding.

The mean (expected value) and variance of the random variable are:

    E[X] = (a + b) / 2
    Var[X] = (b - a) ** 2 / 12

"""

triangular = _inst.triangular
"""Triangular distribution.

Continuous distribution bounded by given lower and upper limits,
and having a given mode value in-between.

http://en.wikipedia.org/wiki/Triangular_distribution

The mean (expected value) and variance of the random variable are:

    E[X] = (low + high + mode) / 3
    Var[X] = (low**2 + high**2 + mode**2 - low*high - low*mode - high*mode) / 18

"""

randint = _inst.randint
"""Return random integer in range [a, b], including both end points."""

choice = _inst.choice
"""Choose a random element from a non-empty sequence."""

randrange = _inst.randrange
"""Choose a random item from range(stop) or range(start, stop[, step]).

Roughly equivalent to ``choice(range(start, stop, step))`` but
supports arbitrarily large ranges and is optimized for common cases.

"""

sample = _inst.sample
"""Chooses k unique random elements from a population sequence.

Returns a new list containing elements from the population while
leaving the original population unchanged.  The resulting list is
in selection order so that all sub-slices will also be valid random
samples.  This allows raffle winners (the sample) to be partitioned
into grand prize and second place winners (the subslices).

Members of the population need not be hashable or unique.  If the
population contains repeats, then each occurrence is a possible
selection in the sample.

Repeated elements can be specified one at a time or with the optional
counts parameter.  For example:

    sample(['red', 'blue'], counts=[4, 2], k=5)

is equivalent to:

    sample(['red', 'red', 'red', 'red', 'blue', 'blue'], k=5)

To choose a sample from a range of integers, use range() for the
population argument.  This is especially fast and space efficient
for sampling from a large population:

    sample(range(10000000), 60)

"""

shuffle = _inst.shuffle
"""Shuffle list x in place, and return None."""

choices = _inst.choices
"""Return a k sized list of population elements chosen with replacement.

If the relative weights or cumulative weights are not specified,
the selections are made with equal probability.

"""

normalvariate = _inst.normalvariate
"""Normal distribution.

mu is the mean, and sigma is the standard deviation.

"""

lognormvariate = _inst.lognormvariate
"""Log normal distribution.

If you take the natural logarithm of this distribution, you'll get a
normal distribution with mean mu and standard deviation sigma.
mu can have any value, and sigma must be greater than zero.

"""

expovariate = _inst.expovariate
"""Exponential distribution.

lambd is 1.0 divided by the desired mean.  It should be
nonzero.  (The parameter would be called "lambda", but that is
a reserved word in Python.)  Returned values range from 0 to
positive infinity if lambd is positive, and from negative
infinity to 0 if lambd is negative.

The mean (expected value) and variance of the random variable are:

    E[X] = 1 / lambd
    Var[X] = 1 / lambd ** 2

"""

vonmisesvariate = _inst.vonmisesvariate
"""Circular data distribution.

mu is the mean angle, expressed in radians between 0 and 2*pi, and
kappa is the concentration parameter, which must be greater than or
equal to zero.  If kappa is equal to zero, this distribution reduces
to a uniform random angle over the range 0 to 2*pi.

"""

gammavariate = _inst.gammavariate
"""Gamma distribution.  Not the gamma function!

Conditions on the parameters are alpha > 0 and beta > 0.

The probability distribution function is:

            x ** (alpha - 1) * math.exp(-x / beta)
  pdf(x) =  --------------------------------------
              math.gamma(alpha) * beta ** alpha

The mean (expected value) and variance of the random variable are:

    E[X] = alpha * beta
    Var[X] = alpha * beta ** 2

"""

gauss = _inst.gauss
"""Gaussian distribution.

mu is the mean, and sigma is the standard deviation.  This is
slightly faster than the normalvariate() function.

Not thread-safe without a lock around calls.

"""

if sys.version_info >= (3, 12):
    binomialvariate = _inst.binomialvariate
    """Binomial random variable.

    Gives the number of successes for *n* independent trials
    with the probability of success in each trial being *p*:

        sum(random() < p for i in range(n))

    Returns an integer in the range:

        0 <= X <= n

    The integer is chosen with the probability:

        P(X == k) = math.comb(n, k) * p ** k * (1 - p) ** (n - k)

    The mean (expected value) and variance of the random variable are:

        E[X] = n * p
        Var[X] = n * p * (1 - p)

    """

betavariate = _inst.betavariate
"""Beta distribution.

Conditions on the parameters are alpha > 0 and beta > 0.
Returned values range between 0 and 1.

The mean (expected value) and variance of the random variable are:

    E[X] = alpha / (alpha + beta)
    Var[X] = alpha * beta / ((alpha + beta)**2 * (alpha + beta + 1))

"""

paretovariate = _inst.paretovariate
"""Pareto distribution.  alpha is the shape parameter."""

weibullvariate = _inst.weibullvariate
"""Weibull distribution.

alpha is the scale parameter and beta is the shape parameter.

"""

getstate = _inst.getstate
"""Return internal state; can be passed to setstate() later."""

setstate = _inst.setstate
"""Restore internal state from object returned by getstate()."""

getrandbits = _inst.getrandbits
randbytes = _inst.randbytes
"""Generate n random bytes."""



import sys
from _typeshed import StrOrBytesPath
from types import FrameType
from typing import Any

from .tasks import Task

def _task_repr_info(task: Task[Any]) -> list[str]: ...  # undocumented

if sys.version_info >= (3, 13):
    def _task_repr(task: Task[Any]) -> str: ...  # undocumented

elif sys.version_info >= (3, 11):
    def _task_repr(self: Task[Any]) -> str: ...  # undocumented

def _task_get_stack(task: Task[Any], limit: int | None) -> list[FrameType]: ...  # undocumented
def _task_print_stack(task: Task[Any], limit: int | None, file: StrOrBytesPath) -> None: ...  # undocumented



"""An RFC 5321 smtp proxy with optional RFC 1870 and RFC 6531 extensions.

Usage: %(program)s [options] [localhost:localport [remotehost:remoteport]]

Options:

    --nosetuid
    -n
        This program generally tries to setuid `nobody', unless this flag is
        set.  The setuid call will fail if this program is not run as root (in
        which case, use this flag).

    --version
    -V
        Print the version number and exit.

    --class classname
    -c classname
        Use `classname' as the concrete SMTP proxy class.  Uses `PureProxy' by
        default.

    --size limit
    -s limit
        Restrict the total size of the incoming message to "limit" number of
        bytes via the RFC 1870 SIZE extension.  Defaults to 33554432 bytes.

    --smtputf8
    -u
        Enable the SMTPUTF8 extension and behave as an RFC 6531 smtp proxy.

    --debug
    -d
        Turn on debugging prints.

    --help
    -h
        Print this message and exit.

Version: %(__version__)s

If localhost is not given then `localhost' is used, and if localport is not
given then 8025 is used.  If remotehost is not given then `localhost' is used,
and if remoteport is not given, then 25 is used.
"""

import asynchat
import asyncore
import socket
import sys
from collections import defaultdict
from typing import Any, TypeAlias
from typing_extensions import deprecated

if sys.version_info >= (3, 11):
    __all__ = ["SMTPChannel", "SMTPServer", "DebuggingServer", "PureProxy"]
else:
    __all__ = ["SMTPChannel", "SMTPServer", "DebuggingServer", "PureProxy", "MailmanProxy"]

_Address: TypeAlias = tuple[str, int]  # (host, port)

class SMTPChannel(asynchat.async_chat):
    COMMAND: int
    DATA: int

    command_size_limits: defaultdict[str, int]
    smtp_server: SMTPServer
    conn: socket.socket
    addr: Any
    received_lines: list[str]
    smtp_state: int
    seen_greeting: str
    mailfrom: str
    rcpttos: list[str]
    received_data: str
    fqdn: str
    peer: str

    command_size_limit: int
    data_size_limit: int

    enable_SMTPUTF8: bool
    @property
    def max_command_size_limit(self) -> int: ...
    def __init__(
        self,
        server: SMTPServer,
        conn: socket.socket,
        addr: Any,
        data_size_limit: int = 33554432,
        map: asyncore._MapType | None = None,
        enable_SMTPUTF8: bool = False,
        decode_data: bool = False,
    ) -> None: ...
    # base asynchat.async_chat.push() accepts bytes
    def push(self, msg: str) -> None: ...  # type: ignore[override]
    def collect_incoming_data(self, data: bytes) -> None: ...
    def found_terminator(self) -> None: ...
    def smtp_HELO(self, arg: str) -> None: ...
    def smtp_NOOP(self, arg: str) -> None: ...
    def smtp_QUIT(self, arg: str) -> None: ...
    def smtp_MAIL(self, arg: str) -> None: ...
    def smtp_RCPT(self, arg: str) -> None: ...
    def smtp_RSET(self, arg: str) -> None: ...
    def smtp_DATA(self, arg: str) -> None: ...
    def smtp_EHLO(self, arg: str) -> None: ...
    def smtp_HELP(self, arg: str) -> None: ...
    def smtp_VRFY(self, arg: str) -> None: ...
    def smtp_EXPN(self, arg: str) -> None: ...

class SMTPServer(asyncore.dispatcher):
    channel_class: type[SMTPChannel]

    data_size_limit: int
    enable_SMTPUTF8: bool
    def __init__(
        self,
        localaddr: _Address,
        remoteaddr: _Address,
        data_size_limit: int = 33554432,
        map: asyncore._MapType | None = None,
        enable_SMTPUTF8: bool = False,
        decode_data: bool = False,
    ) -> None: ...
    def handle_accepted(self, conn: socket.socket, addr: Any) -> None: ...
    def process_message(self, peer: _Address, mailfrom: str, rcpttos: list[str], data: bytes | str, **kwargs: Any) -> str | None:
        """Override this abstract method to handle messages from the client.

        peer is a tuple containing (ipaddr, port) of the client that made the
        socket connection to our smtp port.

        mailfrom is the raw address the client claims the message is coming
        from.

        rcpttos is a list of raw addresses the client wishes to deliver the
        message to.

        data is a string containing the entire full text of the message,
        headers (if supplied) and all.  It has been `de-transparencied'
        according to RFC 821, Section 4.5.2.  In other words, a line
        containing a `.' followed by other text has had the leading dot
        removed.

        kwargs is a dictionary containing additional information.  It is
        empty if decode_data=True was given as init parameter, otherwise
        it will contain the following keys:
            'mail_options': list of parameters to the mail command.  All
                            elements are uppercase strings.  Example:
                            ['BODY=8BITMIME', 'SMTPUTF8'].
            'rcpt_options': same, for the rcpt command.

        This function should return None for a normal `250 Ok' response;
        otherwise, it should return the desired response string in RFC 821
        format.

        """

class DebuggingServer(SMTPServer): ...

class PureProxy(SMTPServer):
    def process_message(self, peer: _Address, mailfrom: str, rcpttos: list[str], data: bytes | str) -> str | None: ...  # type: ignore[override]

if sys.version_info < (3, 11):
    @deprecated("Deprecated since Python 3.9; removed in Python 3.11.")
    class MailmanProxy(PureProxy):
        def process_message(self, peer: _Address, mailfrom: str, rcpttos: list[str], data: bytes | str) -> str | None: ...  # type: ignore[override]



"""A generic class to build line-oriented command interpreters.

Interpreters constructed with this class obey the following conventions:

1. End of file on input is processed as the command 'EOF'.
2. A command is parsed out of each line by collecting the prefix composed
   of characters in the identchars member.
3. A command 'foo' is dispatched to a method 'do_foo()'; the do_ method
   is passed a single argument consisting of the remainder of the line.
4. Typing an empty line repeats the last command.  (Actually, it calls the
   method 'emptyline', which may be overridden in a subclass.)
5. There is a predefined 'help' method.  Given an argument 'topic', it
   calls the command 'help_topic'.  With no arguments, it lists all topics
   with defined help_ functions, broken into up to three topics; documented
   commands, miscellaneous help topics, and undocumented commands.
6. The command '?' is a synonym for 'help'.  The command '!' is a synonym
   for 'shell', if a do_shell method exists.
7. If completion is enabled, completing commands will be done automatically,
   and completing of commands args is done by calling complete_foo() with
   arguments text, line, begidx, endidx.  text is string we are matching
   against, all returned matches must begin with it.  line is the current
   input line (lstripped), begidx and endidx are the beginning and end
   indexes of the text being matched, which could be used to provide
   different completion depending upon which position the argument is in.

The 'default' method may be overridden to intercept commands for which there
is no do_ method.

The 'completedefault' method may be overridden to intercept completions for
commands that have no complete_ method.

The data member 'self.ruler' sets the character used to draw separator lines
in the help messages.  If empty, no ruler line is drawn.  It defaults to "=".

If the value of 'self.intro' is nonempty when the cmdloop method is called,
it is printed out on interpreter startup.  This value may be overridden
via an optional argument to the cmdloop() method.

The data members 'self.doc_header', 'self.misc_header', and
'self.undoc_header' set the headers used for the help function's
listings of documented functions, miscellaneous topics, and undocumented
functions respectively.
"""

from collections.abc import Callable
from typing import IO, Any, Final
from typing_extensions import LiteralString

__all__ = ["Cmd"]

PROMPT: Final = "(Cmd) "
IDENTCHARS: Final[LiteralString]  # Too big to be `Literal`

class Cmd:
    """A simple framework for writing line-oriented command interpreters.

    These are often useful for test harnesses, administrative tools, and
    prototypes that will later be wrapped in a more sophisticated interface.

    A Cmd instance or subclass instance is a line-oriented interpreter
    framework.  There is no good reason to instantiate Cmd itself; rather,
    it's useful as a superclass of an interpreter class you define yourself
    in order to inherit Cmd's methods and encapsulate action methods.

    """

    prompt: str
    identchars: str
    ruler: str
    lastcmd: str
    intro: Any | None
    doc_leader: str
    doc_header: str
    misc_header: str
    undoc_header: str
    nohelp: str
    use_rawinput: bool
    stdin: IO[str]
    stdout: IO[str]
    cmdqueue: list[str]
    completekey: str
    def __init__(self, completekey: str = "tab", stdin: IO[str] | None = None, stdout: IO[str] | None = None) -> None:
        """Instantiate a line-oriented interpreter framework.

        The optional argument 'completekey' is the readline name of a
        completion key; it defaults to the Tab key. If completekey is
        not None and the readline module is available, command completion
        is done automatically. The optional arguments stdin and stdout
        specify alternate input and output file objects; if not specified,
        sys.stdin and sys.stdout are used.

        """

    old_completer: Callable[[str, int], str | None] | None
    def cmdloop(self, intro: Any | None = None) -> None:
        """Repeatedly issue a prompt, accept input, parse an initial prefix
        off the received input, and dispatch to action methods, passing them
        the remainder of the line as argument.

        """

    def precmd(self, line: str) -> str:
        """Hook method executed just before the command line is
        interpreted, but after the input prompt is generated and issued.

        """

    def postcmd(self, stop: bool, line: str) -> bool:
        """Hook method executed just after a command dispatch is finished."""

    def preloop(self) -> None:
        """Hook method executed once when the cmdloop() method is called."""

    def postloop(self) -> None:
        """Hook method executed once when the cmdloop() method is about to
        return.

        """

    def parseline(self, line: str) -> tuple[str | None, str | None, str]:
        """Parse the line into a command name and a string containing
        the arguments.  Returns a tuple containing (command, args, line).
        'command' and 'args' may be None if the line couldn't be parsed.
        """

    def onecmd(self, line: str) -> bool:
        """Interpret the argument as though it had been typed in response
        to the prompt.

        This may be overridden, but should not normally need to be;
        see the precmd() and postcmd() methods for useful execution hooks.
        The return value is a flag indicating whether interpretation of
        commands by the interpreter should stop.

        """

    def emptyline(self) -> bool:
        """Called when an empty line is entered in response to the prompt.

        If this method is not overridden, it repeats the last nonempty
        command entered.

        """

    def default(self, line: str) -> None:
        """Called on an input line when the command prefix is not recognized.

        If this method is not overridden, it prints an error message and
        returns.

        """

    def completedefault(self, *ignored: Any) -> list[str]:
        """Method called to complete an input line when no command-specific
        complete_*() method is available.

        By default, it returns an empty list.

        """

    def completenames(self, text: str, *ignored: Any) -> list[str]: ...
    completion_matches: list[str] | None
    def complete(self, text: str, state: int) -> list[str] | None:
        """Return the next possible completion for 'text'.

        If a command has not been entered, then complete against command list.
        Otherwise try to call complete_<command> to get list of completions.
        """

    def get_names(self) -> list[str]: ...
    # Only the first element of args matters.
    def complete_help(self, *args: Any) -> list[str]: ...
    def do_help(self, arg: str) -> bool | None:
        """List available commands with "help" or detailed help with "help cmd"."""

    def print_topics(self, header: str, cmds: list[str] | None, cmdlen: Any, maxcol: int) -> None: ...
    def columnize(self, list: list[str] | None, displaywidth: int = 80) -> None:
        """Display a list of strings as a compact set of columns.

        Each column is only as wide as necessary.
        Columns are separated by two spaces (one was not legible enough).
        """



"""Mailcap file handling.  See RFC 1524."""

from collections.abc import Mapping, Sequence
from typing import TypeAlias

_Cap: TypeAlias = dict[str, str | int]

__all__ = ["getcaps", "findmatch"]

def findmatch(
    caps: Mapping[str, list[_Cap]], MIMEtype: str, key: str = "view", filename: str = "/dev/null", plist: Sequence[str] = []
) -> tuple[str | None, _Cap | None]:
    """Find a match for a mailcap entry.

    Return a tuple containing the command line, and the mailcap entry
    used; (None, None) if no match is found.  This may invoke the
    'test' command of several matching entries before deciding which
    entry to use.

    """

def getcaps() -> dict[str, list[_Cap]]:
    """Return a dictionary containing the mailcap database.

    The dictionary maps a MIME type (in all lowercase, e.g. 'text/plain')
    to a list of dictionaries corresponding to mailcap entries.  The list
    collects all the entries for that MIME type from all available mailcap
    files.  Each dictionary contains key-value pairs for that MIME type,
    where the viewing command is stored with the key "view".

    """



import sys
from _typeshed import Unused
from collections.abc import Awaitable, Callable, Coroutine
from contextvars import Context
from typing import Any, TypeVar, final
from typing_extensions import Self

from .events import AbstractEventLoop

# Keep asyncio.__all__ updated with any changes to __all__ here
if sys.version_info >= (3, 11):
    __all__ = ("Runner", "run")
else:
    __all__ = ("run",)
_T = TypeVar("_T")

if sys.version_info >= (3, 11):
    @final
    class Runner:
        """A context manager that controls event loop life cycle.

        The context manager always creates a new event loop,
        allows to run async functions inside it,
        and properly finalizes the loop at the context manager exit.

        If debug is True, the event loop will be run in debug mode.
        If loop_factory is passed, it is used for new event loop creation.

        asyncio.run(main(), debug=True)

        is a shortcut for

        with asyncio.Runner(debug=True) as runner:
            runner.run(main())

        The run() method can be called multiple times within the runner's
        context.

        This can be useful for interactive console (e.g. IPython),
        unittest runners, console tools, -- everywhere when async code
        is called from existing sync framework and where the preferred single
        asyncio.run() call doesn't work.

        """

        def __init__(self, *, debug: bool | None = None, loop_factory: Callable[[], AbstractEventLoop] | None = None) -> None: ...
        def __enter__(self) -> Self: ...
        def __exit__(self, exc_type: Unused, exc_val: Unused, exc_tb: Unused) -> None: ...
        def close(self) -> None:
            """Shutdown and close event loop."""

        def get_loop(self) -> AbstractEventLoop:
            """Return embedded event loop."""

        if sys.version_info >= (3, 14):
            def run(self, coro: Awaitable[_T], *, context: Context | None = None) -> _T:
                """Run code in the embedded event loop."""

        else:
            def run(self, coro: Coroutine[Any, Any, _T], *, context: Context | None = None) -> _T:
                """Run a coroutine inside the embedded event loop."""

if sys.version_info >= (3, 14):
    def run(main: Awaitable[_T], *, debug: bool | None = None, loop_factory: Callable[[], AbstractEventLoop] | None = None) -> _T:
        """Execute the coroutine and return the result.

        This function runs the passed coroutine, taking care of
        managing the asyncio event loop, finalizing asynchronous
        generators and closing the default executor.

        This function cannot be called when another asyncio event loop is
        running in the same thread.

        If debug is True, the event loop will be run in debug mode.
        If loop_factory is passed, it is used for new event loop creation.

        This function always creates a new event loop and closes it at the end.
        It should be used as a main entry point for asyncio programs, and should
        ideally only be called once.

        The executor is given a timeout duration of 5 minutes to shutdown.
        If the executor hasn't finished within that duration, a warning is
        emitted and the executor is closed.

        Example:

            async def main():
                await asyncio.sleep(1)
                print('hello')

            asyncio.run(main())
        """

elif sys.version_info >= (3, 12):
    def run(
        main: Coroutine[Any, Any, _T], *, debug: bool | None = None, loop_factory: Callable[[], AbstractEventLoop] | None = None
    ) -> _T:
        """Execute the coroutine and return the result.

        This function runs the passed coroutine, taking care of
        managing the asyncio event loop, finalizing asynchronous
        generators and closing the default executor.

        This function cannot be called when another asyncio event loop is
        running in the same thread.

        If debug is True, the event loop will be run in debug mode.
        If loop_factory is passed, it is used for new event loop creation.

        This function always creates a new event loop and closes it at the end.
        It should be used as a main entry point for asyncio programs, and should
        ideally only be called once.

        The executor is given a timeout duration of 5 minutes to shutdown.
        If the executor hasn't finished within that duration, a warning is
        emitted and the executor is closed.

        Example:

            async def main():
                await asyncio.sleep(1)
                print('hello')

            asyncio.run(main())
        """

else:
    def run(main: Coroutine[Any, Any, _T], *, debug: bool | None = None) -> _T:
        """Execute the coroutine and return the result.

        This function runs the passed coroutine, taking care of
        managing the asyncio event loop and finalizing asynchronous
        generators.

        This function cannot be called when another asyncio event loop is
        running in the same thread.

        If debug is True, the event loop will be run in debug mode.

        This function always creates a new event loop and closes it at the end.
        It should be used as a main entry point for asyncio programs, and should
        ideally only be called once.

        Example:

            async def main():
                await asyncio.sleep(1)
                print('hello')

            asyncio.run(main())
        """
